"""
Tests for the zfs utils library

:codeauthor:    Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:platform:      illumos,freebsd,linux

.. versionadded:: 2018.3.1
"""

from collections import OrderedDict

import salt.utils.zfs as zfs
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase
from tests.support.zfs import ZFSMockData


class ZfsUtilsTestCase(TestCase):
    """
    This class contains a set of functions that test salt.utils.zfs utils
    """

    def setUp(self):
        # property_map mocks
        mock_data = ZFSMockData()
        self.pmap_zfs = mock_data.pmap_zfs
        self.pmap_zpool = mock_data.pmap_zpool
        self.pmap_exec_zfs = mock_data.pmap_exec_zfs
        self.pmap_exec_zpool = mock_data.pmap_exec_zpool
        for name in ("pmap_zfs", "pmap_zpool", "pmap_exec_zfs", "pmap_exec_zpool"):
            self.addCleanup(delattr, self, name)

    # NOTE: test parameter parsing
    def test_is_supported(self):
        """
        Test zfs.is_supported method
        """
        for value in [False, True]:
            with patch("salt.utils.path.which", MagicMock(return_value=value)):
                with patch(
                    "salt.utils.platform.is_linux", MagicMock(return_value=value)
                ):
                    self.assertEqual(value, zfs.is_supported())

    def test_property_data_zpool(self):
        """
        Test parsing of zpool get output
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "_exec", MagicMock(return_value=self.pmap_exec_zpool)
                ):
                    self.assertEqual(zfs.property_data_zpool(), self.pmap_zpool)

    def test_property_data_zfs(self):
        """
        Test parsing of zfs get output
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "_exec", MagicMock(return_value=self.pmap_exec_zfs)
                ):
                    self.assertEqual(zfs.property_data_zfs(), self.pmap_zfs)

    # NOTE: testing from_bool results
    def test_from_bool_on(self):
        """
        Test from_bool with 'on'
        """
        self.assertTrue(zfs.from_bool("on"))
        self.assertTrue(zfs.from_bool(zfs.from_bool("on")))

    def test_from_bool_off(self):
        """
        Test from_bool with 'off'
        """
        self.assertFalse(zfs.from_bool("off"))
        self.assertFalse(zfs.from_bool(zfs.from_bool("off")))

    def test_from_bool_none(self):
        """
        Test from_bool with 'none'
        """
        self.assertEqual(zfs.from_bool("none"), None)
        self.assertEqual(zfs.from_bool(zfs.from_bool("none")), None)

    def test_from_bool_passthrough(self):
        """
        Test from_bool with 'passthrough'
        """
        self.assertEqual(zfs.from_bool("passthrough"), "passthrough")
        self.assertEqual(zfs.from_bool(zfs.from_bool("passthrough")), "passthrough")

    def test_from_bool_alt_yes(self):
        """
        Test from_bool_alt with 'yes'
        """
        self.assertTrue(zfs.from_bool_alt("yes"))
        self.assertTrue(zfs.from_bool_alt(zfs.from_bool_alt("yes")))

    def test_from_bool_alt_no(self):
        """
        Test from_bool_alt with 'no'
        """
        self.assertFalse(zfs.from_bool_alt("no"))
        self.assertFalse(zfs.from_bool_alt(zfs.from_bool_alt("no")))

    def test_from_bool_alt_none(self):
        """
        Test from_bool_alt with 'none'
        """
        self.assertEqual(zfs.from_bool_alt("none"), None)
        self.assertEqual(zfs.from_bool_alt(zfs.from_bool_alt("none")), None)

    def test_from_bool_alt_passthrough(self):
        """
        Test from_bool_alt with 'passthrough'
        """
        self.assertEqual(zfs.from_bool_alt("passthrough"), "passthrough")
        self.assertEqual(
            zfs.from_bool_alt(zfs.from_bool_alt("passthrough")), "passthrough"
        )

    # NOTE: testing to_bool results
    def test_to_bool_true(self):
        """
        Test to_bool with True
        """
        self.assertEqual(zfs.to_bool(True), "on")
        self.assertEqual(zfs.to_bool(zfs.to_bool(True)), "on")

    def test_to_bool_false(self):
        """
        Test to_bool with False
        """
        self.assertEqual(zfs.to_bool(False), "off")
        self.assertEqual(zfs.to_bool(zfs.to_bool(False)), "off")

    def test_to_bool_none(self):
        """
        Test to_bool with None
        """
        self.assertEqual(zfs.to_bool(None), "none")
        self.assertEqual(zfs.to_bool(zfs.to_bool(None)), "none")

    def test_to_bool_passthrough(self):
        """
        Test to_bool with 'passthrough'
        """
        self.assertEqual(zfs.to_bool("passthrough"), "passthrough")
        self.assertEqual(zfs.to_bool(zfs.to_bool("passthrough")), "passthrough")

    def test_to_bool_alt_true(self):
        """
        Test to_bool_alt with True
        """
        self.assertEqual(zfs.to_bool_alt(True), "yes")
        self.assertEqual(zfs.to_bool_alt(zfs.to_bool_alt(True)), "yes")

    def test_to_bool_alt_false(self):
        """
        Test to_bool_alt with False
        """
        self.assertEqual(zfs.to_bool_alt(False), "no")
        self.assertEqual(zfs.to_bool_alt(zfs.to_bool_alt(False)), "no")

    def test_to_bool_alt_none(self):
        """
        Test to_bool_alt with None
        """
        self.assertEqual(zfs.to_bool_alt(None), "none")
        self.assertEqual(zfs.to_bool_alt(zfs.to_bool_alt(None)), "none")

    def test_to_bool_alt_passthrough(self):
        """
        Test to_bool_alt with 'passthrough'
        """
        self.assertEqual(zfs.to_bool_alt("passthrough"), "passthrough")
        self.assertEqual(zfs.to_bool_alt(zfs.to_bool_alt("passthrough")), "passthrough")

    # NOTE: testing from_numeric results
    def test_from_numeric_str(self):
        """
        Test from_numeric with '42'
        """
        self.assertEqual(zfs.from_numeric("42"), 42)
        self.assertEqual(zfs.from_numeric(zfs.from_numeric("42")), 42)

    def test_from_numeric_int(self):
        """
        Test from_numeric with 42
        """
        self.assertEqual(zfs.from_numeric(42), 42)
        self.assertEqual(zfs.from_numeric(zfs.from_numeric(42)), 42)

    def test_from_numeric_none(self):
        """
        Test from_numeric with 'none'
        """
        self.assertEqual(zfs.from_numeric("none"), None)
        self.assertEqual(zfs.from_numeric(zfs.from_numeric("none")), None)

    def test_from_numeric_passthrough(self):
        """
        Test from_numeric with 'passthrough'
        """
        self.assertEqual(zfs.from_numeric("passthrough"), "passthrough")
        self.assertEqual(
            zfs.from_numeric(zfs.from_numeric("passthrough")), "passthrough"
        )

    # NOTE: testing to_numeric results
    def test_to_numeric_str(self):
        """
        Test to_numeric with '42'
        """
        self.assertEqual(zfs.to_numeric("42"), 42)
        self.assertEqual(zfs.to_numeric(zfs.to_numeric("42")), 42)

    def test_to_numeric_int(self):
        """
        Test to_numeric with 42
        """
        self.assertEqual(zfs.to_numeric(42), 42)
        self.assertEqual(zfs.to_numeric(zfs.to_numeric(42)), 42)

    def test_to_numeric_none(self):
        """
        Test to_numeric with 'none'
        """
        self.assertEqual(zfs.to_numeric(None), "none")
        self.assertEqual(zfs.to_numeric(zfs.to_numeric(None)), "none")

    def test_to_numeric_passthrough(self):
        """
        Test to_numeric with 'passthrough'
        """
        self.assertEqual(zfs.to_numeric("passthrough"), "passthrough")
        self.assertEqual(zfs.to_numeric(zfs.to_numeric("passthrough")), "passthrough")

    # NOTE: testing from_size results
    def test_from_size_absolute(self):
        """
        Test from_size with '5G'
        """
        self.assertEqual(zfs.from_size("5G"), 5368709120)
        self.assertEqual(zfs.from_size(zfs.from_size("5G")), 5368709120)

    def test_from_size_decimal(self):
        """
        Test from_size with '4.20M'
        """
        self.assertEqual(zfs.from_size("4.20M"), 4404019)
        self.assertEqual(zfs.from_size(zfs.from_size("4.20M")), 4404019)

    def test_from_size_none(self):
        """
        Test from_size with 'none'
        """
        self.assertEqual(zfs.from_size("none"), None)
        self.assertEqual(zfs.from_size(zfs.from_size("none")), None)

    def test_from_size_passthrough(self):
        """
        Test from_size with 'passthrough'
        """
        self.assertEqual(zfs.from_size("passthrough"), "passthrough")
        self.assertEqual(zfs.from_size(zfs.from_size("passthrough")), "passthrough")

    # NOTE: testing to_size results
    def test_to_size_str_absolute(self):
        """
        Test to_size with '5368709120'
        """
        self.assertEqual(zfs.to_size("5368709120"), "5G")
        self.assertEqual(zfs.to_size(zfs.to_size("5368709120")), "5G")

    def test_to_size_str_decimal(self):
        """
        Test to_size with '4404019'
        """
        self.assertEqual(zfs.to_size("4404019"), "4.20M")
        self.assertEqual(zfs.to_size(zfs.to_size("4404019")), "4.20M")

    def test_to_size_int_absolute(self):
        """
        Test to_size with 5368709120
        """
        self.assertEqual(zfs.to_size(5368709120), "5G")
        self.assertEqual(zfs.to_size(zfs.to_size(5368709120)), "5G")

    def test_to_size_int_decimal(self):
        """
        Test to_size with 4404019
        """
        self.assertEqual(zfs.to_size(4404019), "4.20M")
        self.assertEqual(zfs.to_size(zfs.to_size(4404019)), "4.20M")

    def test_to_size_none(self):
        """
        Test to_size with 'none'
        """
        self.assertEqual(zfs.to_size(None), "none")
        self.assertEqual(zfs.to_size(zfs.to_size(None)), "none")

    def test_to_size_passthrough(self):
        """
        Test to_size with 'passthrough'
        """
        self.assertEqual(zfs.to_size("passthrough"), "passthrough")
        self.assertEqual(zfs.to_size(zfs.to_size("passthrough")), "passthrough")

    # NOTE: testing from_str results
    def test_from_str_space(self):
        """
        Test from_str with "\"my pool/my dataset\"
        """
        self.assertEqual(zfs.from_str('"my pool/my dataset"'), "my pool/my dataset")
        self.assertEqual(
            zfs.from_str(zfs.from_str('"my pool/my dataset"')), "my pool/my dataset"
        )

    def test_from_str_squote_space(self):
        """
        Test from_str with "my pool/jorge's dataset"
        """
        self.assertEqual(
            zfs.from_str("my pool/jorge's dataset"), "my pool/jorge's dataset"
        )
        self.assertEqual(
            zfs.from_str(zfs.from_str("my pool/jorge's dataset")),
            "my pool/jorge's dataset",
        )

    def test_from_str_dquote_space(self):
        """
        Test from_str with "my pool/the \"good\" stuff"
        """
        self.assertEqual(
            zfs.from_str('my pool/the "good" stuff'), 'my pool/the "good" stuff'
        )
        self.assertEqual(
            zfs.from_str(zfs.from_str('my pool/the "good" stuff')),
            'my pool/the "good" stuff',
        )

    def test_from_str_none(self):
        """
        Test from_str with 'none'
        """
        self.assertEqual(zfs.from_str("none"), None)
        self.assertEqual(zfs.from_str(zfs.from_str("none")), None)

    def test_from_str_passthrough(self):
        """
        Test from_str with 'passthrough'
        """
        self.assertEqual(zfs.from_str("passthrough"), "passthrough")
        self.assertEqual(zfs.from_str(zfs.from_str("passthrough")), "passthrough")

    # NOTE: testing to_str results
    def test_to_str_space(self):
        """
        Test to_str with 'my pool/my dataset'
        """
        # NOTE: for fun we use both the '"str"' and "\"str\"" way of getting the literal string: "str"
        self.assertEqual(zfs.to_str("my pool/my dataset"), '"my pool/my dataset"')
        self.assertEqual(
            zfs.to_str(zfs.to_str("my pool/my dataset")), '"my pool/my dataset"'
        )

    def test_to_str_squote_space(self):
        """
        Test to_str with "my pool/jorge's dataset"
        """
        self.assertEqual(
            zfs.to_str("my pool/jorge's dataset"), '"my pool/jorge\'s dataset"'
        )
        self.assertEqual(
            zfs.to_str(zfs.to_str("my pool/jorge's dataset")),
            '"my pool/jorge\'s dataset"',
        )

    def test_to_str_none(self):
        """
        Test to_str with 'none'
        """
        self.assertEqual(zfs.to_str(None), "none")
        self.assertEqual(zfs.to_str(zfs.to_str(None)), "none")

    def test_to_str_passthrough(self):
        """
        Test to_str with 'passthrough'
        """
        self.assertEqual(zfs.to_str("passthrough"), "passthrough")
        self.assertEqual(zfs.to_str(zfs.to_str("passthrough")), "passthrough")

    # NOTE: testing is_snapshot
    def test_is_snapshot_snapshot(self):
        """
        Test is_snapshot with a valid snapshot name
        """
        self.assertTrue(zfs.is_snapshot("zpool_name/dataset@backup"))

    def test_is_snapshot_bookmark(self):
        """
        Test is_snapshot with a valid bookmark name
        """
        self.assertFalse(zfs.is_snapshot("zpool_name/dataset#backup"))

    def test_is_snapshot_filesystem(self):
        """
        Test is_snapshot with a valid filesystem name
        """
        self.assertFalse(zfs.is_snapshot("zpool_name/dataset"))

    # NOTE: testing is_bookmark
    def test_is_bookmark_snapshot(self):
        """
        Test is_bookmark with a valid snapshot name
        """
        self.assertFalse(zfs.is_bookmark("zpool_name/dataset@backup"))

    def test_is_bookmark_bookmark(self):
        """
        Test is_bookmark with a valid bookmark name
        """
        self.assertTrue(zfs.is_bookmark("zpool_name/dataset#backup"))

    def test_is_bookmark_filesystem(self):
        """
        Test is_bookmark with a valid filesystem name
        """
        self.assertFalse(zfs.is_bookmark("zpool_name/dataset"))

    # NOTE: testing is_dataset
    def test_is_dataset_snapshot(self):
        """
        Test is_dataset with a valid snapshot name
        """
        self.assertFalse(zfs.is_dataset("zpool_name/dataset@backup"))

    def test_is_dataset_bookmark(self):
        """
        Test is_dataset with a valid bookmark name
        """
        self.assertFalse(zfs.is_dataset("zpool_name/dataset#backup"))

    def test_is_dataset_filesystem(self):
        """
        Test is_dataset with a valid filesystem/volume name
        """
        self.assertTrue(zfs.is_dataset("zpool_name/dataset"))

    # NOTE: testing zfs_command
    def test_zfs_command_simple(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        self.assertEqual(zfs.zfs_command("list"), "/sbin/zfs list")

    def test_zfs_command_none_target(self):
        """
        Test if zfs_command builds the correct string with a target of None
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        self.assertEqual(
                            zfs.zfs_command("list", target=[None, "mypool", None]),
                            "/sbin/zfs list mypool",
                        )

    def test_zfs_command_flag(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_flags = [
                            "-r",  # recursive
                        ]
                        self.assertEqual(
                            zfs.zfs_command("list", flags=my_flags), "/sbin/zfs list -r"
                        )

    def test_zfs_command_opt(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_opts = {
                            "-t": "snap",  # only list snapshots
                        }
                        self.assertEqual(
                            zfs.zfs_command("list", opts=my_opts),
                            "/sbin/zfs list -t snap",
                        )

    def test_zfs_command_flag_opt(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_flags = [
                            "-r",  # recursive
                        ]
                        my_opts = {
                            "-t": "snap",  # only list snapshots
                        }
                        self.assertEqual(
                            zfs.zfs_command("list", flags=my_flags, opts=my_opts),
                            "/sbin/zfs list -r -t snap",
                        )

    def test_zfs_command_target(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_flags = [
                            "-r",  # recursive
                        ]
                        my_opts = {
                            "-t": "snap",  # only list snapshots
                        }
                        self.assertEqual(
                            zfs.zfs_command(
                                "list", flags=my_flags, opts=my_opts, target="mypool"
                            ),
                            "/sbin/zfs list -r -t snap mypool",
                        )

    def test_zfs_command_target_with_space(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_flags = [
                            "-r",  # recursive
                        ]
                        my_opts = {
                            "-t": "snap",  # only list snapshots
                        }
                        self.assertEqual(
                            zfs.zfs_command(
                                "list", flags=my_flags, opts=my_opts, target="my pool"
                            ),
                            '/sbin/zfs list -r -t snap "my pool"',
                        )

    def test_zfs_command_property(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        self.assertEqual(
                            zfs.zfs_command(
                                "get", property_name="quota", target="mypool"
                            ),
                            "/sbin/zfs get quota mypool",
                        )

    def test_zfs_command_property_value(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_flags = [
                            "-r",  # recursive
                        ]
                        self.assertEqual(
                            zfs.zfs_command(
                                "set",
                                flags=my_flags,
                                property_name="quota",
                                property_value="5G",
                                target="mypool",
                            ),
                            "/sbin/zfs set -r quota=5368709120 mypool",
                        )

    def test_zfs_command_multi_property_value(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        property_name = ["quota", "readonly"]
                        property_value = ["5G", "no"]
                        self.assertEqual(
                            zfs.zfs_command(
                                "set",
                                property_name=property_name,
                                property_value=property_value,
                                target="mypool",
                            ),
                            "/sbin/zfs set quota=5368709120 readonly=off mypool",
                        )

    def test_zfs_command_fs_props(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_flags = [
                            "-p",  # create parent
                        ]
                        my_props = {
                            "quota": "1G",
                            "compression": "lz4",
                        }
                        self.assertEqual(
                            zfs.zfs_command(
                                "create",
                                flags=my_flags,
                                filesystem_properties=my_props,
                                target="mypool/dataset",
                            ),
                            "/sbin/zfs create -p -o compression=lz4 -o quota=1073741824"
                            " mypool/dataset",
                        )

    def test_zfs_command_fs_props_with_space(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_props = {
                            "quota": "4.2M",
                            "compression": "lz4",
                        }
                        self.assertEqual(
                            zfs.zfs_command(
                                "create",
                                filesystem_properties=my_props,
                                target="my pool/jorge's dataset",
                            ),
                            '/sbin/zfs create -o compression=lz4 -o quota=4404019 "my'
                            " pool/jorge's dataset\"",
                        )

    # NOTE: testing zpool_command
    def test_zpool_command_simple(self):
        """
        Test if zfs_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        self.assertEqual(zfs.zpool_command("list"), "/sbin/zpool list")

    def test_zpool_command_opt(self):
        """
        Test if zpool_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_opts = {
                            "-o": "name,size",  # show only name and size
                        }
                        self.assertEqual(
                            zfs.zpool_command("list", opts=my_opts),
                            "/sbin/zpool list -o name,size",
                        )

    def test_zpool_command_opt_list(self):
        """
        Test if zpool_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_opts = {
                            "-d": ["/tmp", "/zvol"],
                        }
                        self.assertEqual(
                            zfs.zpool_command("import", opts=my_opts, target="mypool"),
                            "/sbin/zpool import -d /tmp -d /zvol mypool",
                        )

    def test_zpool_command_flag_opt(self):
        """
        Test if zpool_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_opts = {
                            "-o": "name,size",  # show only name and size
                        }
                        self.assertEqual(
                            zfs.zpool_command("list", opts=my_opts),
                            "/sbin/zpool list -o name,size",
                        )

    def test_zpool_command_target(self):
        """
        Test if zpool_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_opts = {
                            "-o": "name,size",  # show only name and size
                        }
                        self.assertEqual(
                            zfs.zpool_command("list", opts=my_opts, target="mypool"),
                            "/sbin/zpool list -o name,size mypool",
                        )

    def test_zpool_command_target_with_space(self):
        """
        Test if zpool_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        fs_props = {
                            "quota": "100G",
                        }
                        pool_props = {
                            "comment": "jorge's comment has a space",
                        }
                        self.assertEqual(
                            zfs.zpool_command(
                                "create",
                                pool_properties=pool_props,
                                filesystem_properties=fs_props,
                                target="my pool",
                            ),
                            "/sbin/zpool create -O quota=107374182400 -o"
                            ' comment="jorge\'s comment has a space" "my pool"',
                        )

    def test_zpool_command_property(self):
        """
        Test if zpool_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        self.assertEqual(
                            zfs.zpool_command(
                                "get", property_name="comment", target="mypool"
                            ),
                            "/sbin/zpool get comment mypool",
                        )

    def test_zpool_command_property_value(self):
        """
        Test if zpool_command builds the correct string
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        my_flags = [
                            "-v",  # verbose
                        ]
                        self.assertEqual(
                            zfs.zpool_command(
                                "iostat", flags=my_flags, target=["mypool", 60, 1]
                            ),
                            "/sbin/zpool iostat -v mypool 60 1",
                        )

    def test_parse_command_result_success(self):
        """
        Test if parse_command_result returns the expected result
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        res = {}
                        res["retcode"] = 0
                        res["stderr"] = ""
                        res["stdout"] = ""
                        self.assertEqual(
                            zfs.parse_command_result(res, "tested"),
                            OrderedDict([("tested", True)]),
                        )

    def test_parse_command_result_success_nolabel(self):
        """
        Test if parse_command_result returns the expected result
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        res = {}
                        res["retcode"] = 0
                        res["stderr"] = ""
                        res["stdout"] = ""
                        self.assertEqual(
                            zfs.parse_command_result(res),
                            OrderedDict(),
                        )

    def test_parse_command_result_fail(self):
        """
        Test if parse_command_result returns the expected result on failure
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        res = {}
                        res["retcode"] = 1
                        res["stderr"] = ""
                        res["stdout"] = ""
                        self.assertEqual(
                            zfs.parse_command_result(res, "tested"),
                            OrderedDict([("tested", False)]),
                        )

    def test_parse_command_result_nolabel(self):
        """
        Test if parse_command_result returns the expected result on failure
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        res = {}
                        res["retcode"] = 1
                        res["stderr"] = ""
                        res["stdout"] = ""
                        self.assertEqual(
                            zfs.parse_command_result(res),
                            OrderedDict(),
                        )

    def test_parse_command_result_fail_message(self):
        """
        Test if parse_command_result returns the expected result on failure with stderr
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        res = {}
                        res["retcode"] = 1
                        res["stderr"] = "\n".join(
                            ["ice is not hot", "usage:", "this should not be printed"]
                        )
                        res["stdout"] = ""
                        self.assertEqual(
                            zfs.parse_command_result(res, "tested"),
                            OrderedDict(
                                [("tested", False), ("error", "ice is not hot")]
                            ),
                        )

    def test_parse_command_result_fail_message_nolabel(self):
        """
        Test if parse_command_result returns the expected result on failure with stderr
        """
        with patch.object(zfs, "_zfs_cmd", MagicMock(return_value="/sbin/zfs")):
            with patch.object(zfs, "_zpool_cmd", MagicMock(return_value="/sbin/zpool")):
                with patch.object(
                    zfs, "property_data_zfs", MagicMock(return_value=self.pmap_zfs)
                ):
                    with patch.object(
                        zfs,
                        "property_data_zpool",
                        MagicMock(return_value=self.pmap_zpool),
                    ):
                        res = {}
                        res["retcode"] = 1
                        res["stderr"] = "\n".join(
                            ["ice is not hot", "usage:", "this should not be printed"]
                        )
                        res["stdout"] = ""
                        self.assertEqual(
                            zfs.parse_command_result(res),
                            OrderedDict([("error", "ice is not hot")]),
                        )



import os

import pytest

import salt.crypt
import salt.utils.stringutils
from tests.support.mock import MagicMock, mock_open, patch

from . import MSG, PRIVKEY_DATA, PUBKEY_DATA, SIG, SIGNATURE

M2Crypto = pytest.importorskip("M2Crypto")


@pytest.mark.slow_test
def test_gen_keys():
    with patch("os.umask", MagicMock()), patch("os.chmod", MagicMock()), patch(
        "os.access", MagicMock(return_value=True)
    ):
        with patch("M2Crypto.RSA.RSA.save_pem", MagicMock()) as save_pem, patch(
            "M2Crypto.RSA.RSA.save_pub_key", MagicMock()
        ) as save_pub:
            with patch("os.path.isfile", return_value=True):
                assert (
                    salt.crypt.gen_keys("/keydir", "keyname", 2048)
                    == f"/keydir{os.sep}keyname.pem"
                )
                save_pem.assert_not_called()
                save_pub.assert_not_called()

            with patch("os.path.isfile", return_value=False):
                assert (
                    salt.crypt.gen_keys("/keydir", "keyname", 2048)
                    == f"/keydir{os.sep}keyname.pem"
                )
                save_pem.assert_called_once_with(
                    f"/keydir{os.sep}keyname.pem", cipher=None
                )
                save_pub.assert_called_once_with(f"/keydir{os.sep}keyname.pub")


def test_sign_message():
    key = M2Crypto.RSA.load_key_string(salt.utils.stringutils.to_bytes(PRIVKEY_DATA))
    with patch("salt.crypt.get_rsa_key", return_value=key):
        assert SIG == salt.crypt.sign_message("/keydir/keyname.pem", MSG)


def test_sign_message_with_passphrase():
    key = M2Crypto.RSA.load_key_string(salt.utils.stringutils.to_bytes(PRIVKEY_DATA))
    with patch("salt.crypt.get_rsa_key", return_value=key):
        assert SIG == salt.crypt.sign_message(
            "/keydir/keyname.pem", MSG, passphrase="password"
        )


def test_verify_signature():
    with patch(
        "salt.utils.files.fopen",
        mock_open(read_data=salt.utils.stringutils.to_bytes(PUBKEY_DATA)),
    ):
        assert salt.crypt.verify_signature("/keydir/keyname.pub", MSG, SIG)


def test_encrypt_decrypt_bin():
    priv_key = M2Crypto.RSA.load_key_string(
        salt.utils.stringutils.to_bytes(PRIVKEY_DATA)
    )
    pub_key = M2Crypto.RSA.load_pub_key_bio(
        M2Crypto.BIO.MemoryBuffer(salt.utils.stringutils.to_bytes(PUBKEY_DATA))
    )
    encrypted = salt.crypt.private_encrypt(priv_key, b"salt")
    decrypted = salt.crypt.public_decrypt(pub_key, encrypted)
    assert b"salt" == decrypted


def test_m2crypto_verify_bytes_47124():
    message = salt.utils.stringutils.to_unicode("meh")
    with patch(
        "salt.utils.files.fopen",
        mock_open(read_data=salt.utils.stringutils.to_bytes(PUBKEY_DATA)),
    ):
        salt.crypt.verify_signature("/keydir/keyname.pub", message, SIGNATURE)


def test_m2_bad_key(key_to_test):
    """
    Load public key with an invalid header using m2crypto and validate it
    """
    key = salt.crypt.get_rsa_pub_key(key_to_test)
    assert key.check_key() == 1


@pytest.mark.slow_test
def test_m2_gen_keys():
    with patch("os.umask", MagicMock()), patch("os.chmod", MagicMock()), patch(
        "os.access", MagicMock(return_value=True)
    ):
        with patch("M2Crypto.RSA.RSA.save_pem", MagicMock()) as save_pem, patch(
            "M2Crypto.RSA.RSA.save_pub_key", MagicMock()
        ) as save_pub:
            with patch("os.path.isfile", return_value=True):
                assert (
                    salt.crypt.gen_keys("/keydir", "keyname", 2048)
                    == f"/keydir{os.sep}keyname.pem"
                )
                save_pem.assert_not_called()
                save_pub.assert_not_called()

            with patch("os.path.isfile", return_value=False):
                assert (
                    salt.crypt.gen_keys("/keydir", "keyname", 2048)
                    == f"/keydir{os.sep}keyname.pem"
                )
                save_pem.assert_called_once_with(
                    f"/keydir{os.sep}keyname.pem", cipher=None
                )
                save_pub.assert_called_once_with(f"/keydir{os.sep}keyname.pub")


def test_m2_sign_message_with_passphrase():
    key = M2Crypto.RSA.load_key_string(salt.utils.stringutils.to_bytes(PRIVKEY_DATA))
    with patch("salt.crypt.get_rsa_key", return_value=key):
        assert SIG == salt.crypt.sign_message(
            "/keydir/keyname.pem", MSG, passphrase="password"
        )


def test_m2_verify_signature():
    with patch(
        "salt.utils.files.fopen",
        mock_open(read_data=salt.utils.stringutils.to_bytes(PUBKEY_DATA)),
    ):
        assert salt.crypt.verify_signature("/keydir/keyname.pub", MSG, SIG)


def test_m2_encrypt_decrypt_bin():
    priv_key = M2Crypto.RSA.load_key_string(
        salt.utils.stringutils.to_bytes(PRIVKEY_DATA)
    )
    pub_key = M2Crypto.RSA.load_pub_key_bio(
        M2Crypto.BIO.MemoryBuffer(salt.utils.stringutils.to_bytes(PUBKEY_DATA))
    )
    encrypted = salt.crypt.private_encrypt(priv_key, b"salt")
    decrypted = salt.crypt.public_decrypt(pub_key, encrypted)
    assert b"salt" == decrypted


def test_m2crypto_verify_unicode_47124():
    message = salt.utils.stringutils.to_bytes("meh")
    with patch(
        "salt.utils.files.fopen",
        mock_open(read_data=salt.utils.stringutils.to_bytes(PUBKEY_DATA)),
    ):
        salt.crypt.verify_signature("/keydir/keyname.pub", message, SIGNATURE)


def test_m2crypto_sign_bytes_47124():
    message = salt.utils.stringutils.to_unicode("meh")
    key = M2Crypto.RSA.load_key_string(salt.utils.stringutils.to_bytes(PRIVKEY_DATA))
    with patch("salt.crypt.get_rsa_key", return_value=key):
        signature = salt.crypt.sign_message(
            "/keydir/keyname.pem", message, passphrase="password"
        )
    assert SIGNATURE == signature


def test_m2crypto_sign_unicode_47124():
    message = salt.utils.stringutils.to_bytes("meh")
    key = M2Crypto.RSA.load_key_string(salt.utils.stringutils.to_bytes(PRIVKEY_DATA))
    with patch("salt.crypt.get_rsa_key", return_value=key):
        signature = salt.crypt.sign_message(
            "/keydir/keyname.pem", message, passphrase="password"
        )
    assert SIGNATURE == signature



import io
import os

import pytest

import salt.modules.cmdmod as cmdmod
import salt.modules.restartcheck as restartcheck
import salt.modules.system as system
import salt.modules.systemd_service as service
import salt.utils.path
from tests.support.mock import ANY, MagicMock, create_autospec, patch
from tests.support.paths import SALT_CODE_DIR


@pytest.fixture
def configure_loader_modules():
    return {restartcheck: {}}


def test_when_timestamp_file_does_not_exist_then_file_changed_nilrt_should_be_True():
    expected_changed = True

    def timestamp_not_exists(filename):
        if filename.endswith(".timestamp"):
            return False
        return True

    with patch("os.path.exists", side_effect=timestamp_not_exists, autospec=True):
        actual_changed = restartcheck._file_changed_nilrt(full_filepath="fnord")
        assert actual_changed == expected_changed


def test_when_timestamp_file_exists_but_not_md5sum_file_then_file_changed_nilrt_should_be_True():
    expected_changed = True

    def timestamp_not_exists(filename):
        if filename.endswith(".md5sum"):
            return False
        return True

    with patch("os.path.exists", side_effect=timestamp_not_exists, autospec=True):
        actual_changed = restartcheck._file_changed_nilrt(full_filepath="fnord")
        assert actual_changed == expected_changed


def test_when_nisysapi_path_exists_and_nilrt_files_changed_then_sysapi_changed_nilrt_should_be_True():
    expected_change = True
    patch_os_path = patch("os.path.exists", return_value=True, autospec=True)
    patch_file_changed = patch(
        "salt.modules.restartcheck._file_changed_nilrt",
        autospec=True,
        return_value=True,
    )
    with patch_os_path, patch_file_changed:
        actual_change = restartcheck._sysapi_changed_nilrt()
    assert actual_change == expected_change


# TODO: can I parametrize a fixture? And use mocked_grain or something? -W. Werner, 2020-08-18
@pytest.mark.parametrize("cpuarch_grain", ["arm", "x86_64"])
def test_when_nisysapi_conf_d_path_does_not_exist_then_sysapi_changed_should_be_False(
    cpuarch_grain,
):
    expected_change = False

    def conf_d_not_exists(filename):
        return "nisysapi/conf.d" not in filename

    patch_os_path = patch(
        "os.path.exists", side_effect=conf_d_not_exists, autospec=True
    )
    patch_file_changed = patch(
        "salt.modules.restartcheck._file_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_grain = patch.dict(restartcheck.__grains__, {"cpuarch": cpuarch_grain})
    with patch_os_path, patch_file_changed, patch_grain:
        actual_change = restartcheck._sysapi_changed_nilrt()
    assert actual_change == expected_change


@pytest.mark.parametrize("cpuarch_grain", ["arm", "x86_64"])
def test_when_nisysapi_conf_d_path_does_exist_and_no_restart_check_file_exists_then_sysapi_changed_should_be_True(
    cpuarch_grain,
):
    expected_change = True

    def conf_d_not_exists(filename):
        return not filename.endswith("/sysapi.conf.d.count")

    patch_os_path = patch(
        "os.path.exists", side_effect=conf_d_not_exists, autospec=True
    )
    patch_file_changed = patch(
        "salt.modules.restartcheck._file_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_grain = patch.dict(restartcheck.__grains__, {"cpuarch": cpuarch_grain})
    with patch_os_path, patch_file_changed, patch_grain:
        actual_change = restartcheck._sysapi_changed_nilrt()
    assert actual_change == expected_change


@pytest.mark.parametrize("cpuarch_grain", ["arm", "x86_64"])
def test_when_nisysapi_conf_d_path_does_exist_and_count_file_exists_and_count_is_different_than_files_in_conf_d_path_then_sysapi_changed_should_be_True(
    cpuarch_grain,
):
    expected_change = True

    # Fake count and listdir should be different values
    fake_count = io.StringIO("42")
    patch_listdir = patch("os.listdir", autospec=True, return_value=["boop"])
    patch_os_path = patch("os.path.exists", return_value=True, autospec=True)
    patch_file_changed = patch(
        "salt.modules.restartcheck._file_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_grain = patch.dict(restartcheck.__grains__, {"cpuarch": cpuarch_grain})
    patch_fopen = patch(
        "salt.utils.files.fopen", autospec=True, return_value=fake_count
    )
    with patch_os_path, patch_file_changed, patch_grain, patch_listdir, patch_fopen:
        actual_change = restartcheck._sysapi_changed_nilrt()
    assert actual_change == expected_change


@pytest.mark.parametrize("cpuarch_grain", ["arm", "x86_64"])
def test_when_nisysapi_conf_d_path_does_exist_and_count_file_exists_and_count_is_same_as_files_in_conf_d_path_but_no_nilrt_files_changed_then_sysapi_changed_should_be_False(
    cpuarch_grain,
):
    expected_change = False
    # listdir should return the same number of values as count
    fake_count = io.StringIO("42")
    patch_listdir = patch("os.listdir", autospec=True, return_value=["boop"] * 42)
    patch_os_path = patch("os.path.exists", return_value=True, autospec=True)
    patch_file_changed = patch(
        "salt.modules.restartcheck._file_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_grain = patch.dict(restartcheck.__grains__, {"cpuarch": cpuarch_grain})
    patch_fopen = patch(
        "salt.utils.files.fopen", autospec=True, return_value=fake_count
    )
    with patch_os_path, patch_file_changed, patch_grain, patch_listdir, patch_fopen:
        actual_change = restartcheck._sysapi_changed_nilrt()
    assert actual_change == expected_change


@pytest.mark.parametrize("cpuarch_grain", ["arm", "x86_64"])
def test_when_nisysapi_conf_d_path_does_exist_and_count_file_exists_and_count_is_same_as_files_in_conf_d_path_and_file_changed_nilrt_then_sysapi_changed_should_be_True(
    cpuarch_grain,
):
    expected_change = True

    def fake_file_changed(filename):
        return filename != "/usr/local/natinst/share/nisysapi.ini"

    # listdir should return the same number of values as count
    fake_count = io.StringIO("42")
    patch_listdir = patch("os.listdir", autospec=True, return_value=["boop"] * 42)
    patch_os_path = patch("os.path.exists", return_value=True, autospec=True)
    patch_file_changed = patch(
        "salt.modules.restartcheck._file_changed_nilrt",
        autospec=True,
        side_effect=fake_file_changed,
    )
    patch_grain = patch.dict(restartcheck.__grains__, {"cpuarch": cpuarch_grain})
    patch_fopen = patch(
        "salt.utils.files.fopen", autospec=True, return_value=fake_count
    )
    with patch_os_path, patch_file_changed, patch_grain, patch_listdir, patch_fopen:
        actual_change = restartcheck._sysapi_changed_nilrt()
    assert actual_change == expected_change


def test_when_nilinuxrt_and_not_kernel_modules_changed_or_sysapi_files_changed_and_not_reboot_required_witnessed_then_no_reboot_should_be_required():
    expected_result = "No packages seem to need to be restarted."
    restart_required = False
    current_kernel = "fnord"

    patch_grains = patch.dict(restartcheck.__grains__, {"os_family": "NILinuxRT"})
    patch_kernel_versions = patch(
        "salt.modules.restartcheck._kernel_versions_nilrt",
        autospec=True,
        return_value=[current_kernel],
    )
    patch_salt = patch.dict(
        restartcheck.__salt__,
        {
            "cmd.run": create_autospec(cmdmod.run, return_value=current_kernel),
            "system.get_reboot_required_witnessed": create_autospec(
                system.get_reboot_required_witnessed,
                return_value=restart_required,
            ),
            "service.get_running": create_autospec(
                service.get_running, return_value=[]
            ),
        },
    )
    patch_kernel_mod_changed = patch(
        "salt.modules.restartcheck._kernel_modules_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_sysapi_changed = patch(
        "salt.modules.restartcheck._sysapi_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_del_files = patch(
        "salt.modules.restartcheck._deleted_files",
        autospec=True,
        return_value=[],
    )

    with (
        patch_grains
    ), (
        patch_kernel_versions
    ), patch_salt, patch_sysapi_changed, patch_kernel_mod_changed, patch_del_files:
        actual_result = restartcheck.restartcheck()
    assert actual_result == expected_result


def test_when_nilinuxrt_and_not_kernel_modules_changed_or_sysapi_files_changed_and_reboot_required_witnessed_then_reboot_should_be_required():
    expected_result = "System restart required.\n\n"
    restart_required = True
    current_kernel = "fnord"

    patch_grains = patch.dict(restartcheck.__grains__, {"os_family": "NILinuxRT"})
    patch_kernel_versions = patch(
        "salt.modules.restartcheck._kernel_versions_nilrt",
        autospec=True,
        return_value=[current_kernel],
    )
    patch_salt = patch.dict(
        restartcheck.__salt__,
        {
            "cmd.run": create_autospec(cmdmod.run, return_value=current_kernel),
            "system.get_reboot_required_witnessed": create_autospec(
                system.get_reboot_required_witnessed,
                return_value=restart_required,
            ),
            "service.get_running": create_autospec(
                service.get_running, return_value=[]
            ),
        },
    )
    patch_kernel_mod_changed = patch(
        "salt.modules.restartcheck._kernel_modules_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_sysapi_changed = patch(
        "salt.modules.restartcheck._sysapi_changed_nilrt",
        autospec=True,
        return_value=False,
    )
    patch_del_files = patch(
        "salt.modules.restartcheck._deleted_files",
        autospec=True,
        return_value=[],
    )

    with (
        patch_grains
    ), (
        patch_kernel_versions
    ), patch_salt, patch_sysapi_changed, patch_kernel_mod_changed, patch_del_files:
        actual_result = restartcheck.restartcheck()
    assert actual_result == expected_result


def test_kernel_versions_debian():
    """
    Test kernel version debian
    """
    mock = MagicMock(return_value="  Installed: 4.9.82-1+deb9u3")
    with patch.dict(restartcheck.__grains__, {"os": "Debian"}):
        with patch.dict(restartcheck.__salt__, {"cmd.run": mock}):
            assert restartcheck._kernel_versions_debian() == ["4.9.82-1+deb9u3"]


def test_kernel_versions_ubuntu():
    """
    Test kernel version ubuntu
    """
    mock = MagicMock(return_value="  Installed: 4.10.0-42.46")
    with patch.dict(restartcheck.__grains__, {"os": "Ubuntu"}):
        with patch.dict(restartcheck.__salt__, {"cmd.run": mock}):
            assert restartcheck._kernel_versions_debian() == [
                "4.10.0-42.46",
                "4.10.0-42-generic #46",
                "4.10.0-42-lowlatency #46",
            ]


def test_kernel_versions_redhat():
    """
    Test if it return a data structure of the current, in-memory rules
    """
    mock = MagicMock(
        return_value=(
            "kernel-3.10.0-862.el7.x86_64                  Thu Apr 5 00:40:00 2018"
        )
    )
    with patch.dict(restartcheck.__salt__, {"cmd.run": mock}):
        assert restartcheck._kernel_versions_redhat() == ["3.10.0-862.el7.x86_64"]


def test_valid_deleted_file_deleted():
    """
    Test (deleted) file
    """
    assert restartcheck._valid_deleted_file("/usr/lib/test (deleted)")


def test_valid_deleted_file_psth_inode():
    """
    Test (path inode=1) file
    """
    assert restartcheck._valid_deleted_file("/usr/lib/test (path inode=1)")


def test_valid_deleted_file_var_log():
    """
    Test /var/log/
    """
    assert not restartcheck._valid_deleted_file("/var/log/test")
    assert not restartcheck._valid_deleted_file("/var/log/test (deleted)")
    assert not restartcheck._valid_deleted_file("/var/log/test (path inode=1)")


def test_valid_deleted_file_var_local_log():
    """
    Test /var/local/log/
    """
    assert not restartcheck._valid_deleted_file("/var/local/log/test")
    assert not restartcheck._valid_deleted_file("/var/local/log/test (deleted)")
    assert not restartcheck._valid_deleted_file("/var/local/log/test (path inode=1)")


def test_valid_deleted_file_var_run():
    """
    Test /var/run/
    """
    assert not restartcheck._valid_deleted_file("/var/run/test")
    assert not restartcheck._valid_deleted_file("/var/run/test (deleted)")
    assert not restartcheck._valid_deleted_file("/var/run/test (path inode=1)")


def test_valid_deleted_file_var_local_run():
    """
    Test /var/local/run/
    """
    assert not restartcheck._valid_deleted_file("/var/local/run/test")
    assert not restartcheck._valid_deleted_file("/var/local/run/test (deleted)")
    assert not restartcheck._valid_deleted_file("/var/local/run/test (path inode=1)")


def test_valid_deleted_file_tmp():
    """
    Test /tmp/
    """
    assert not restartcheck._valid_deleted_file("/tmp/test")
    assert not restartcheck._valid_deleted_file("/tmp/test (deleted)")
    assert not restartcheck._valid_deleted_file("/tmp/test (path inode=1)")


def test_valid_deleted_file_dev_shm():
    """
    Test /dev/shm/
    """
    assert not restartcheck._valid_deleted_file("/dev/shm/test")
    assert not restartcheck._valid_deleted_file("/dev/shm/test (deleted)")
    assert not restartcheck._valid_deleted_file("/dev/shm/test (path inode=1)")


def test_valid_deleted_file_run():
    """
    Test /run/
    """
    assert not restartcheck._valid_deleted_file("/run/test")
    assert not restartcheck._valid_deleted_file("/run/test (deleted)")
    assert not restartcheck._valid_deleted_file("/run/test (path inode=1)")


def test_valid_deleted_file_drm():
    """
    Test /drm/
    """
    assert not restartcheck._valid_deleted_file("/drm/test")
    assert not restartcheck._valid_deleted_file("/drm/test (deleted)")
    assert not restartcheck._valid_deleted_file("/drm/test (path inode=1)")


def test_valid_deleted_file_var_tmp():
    """
    Test /var/tmp/
    """
    assert not restartcheck._valid_deleted_file("/var/tmp/test")
    assert not restartcheck._valid_deleted_file("/var/tmp/test (deleted)")
    assert not restartcheck._valid_deleted_file("/var/tmp/test (path inode=1)")


def test_valid_deleted_file_var_local_tmp():
    """
    Test /var/local/tmp/
    """
    assert not restartcheck._valid_deleted_file("/var/local/tmp/test")
    assert not restartcheck._valid_deleted_file("/var/local/tmp/test (deleted)")
    assert not restartcheck._valid_deleted_file("/var/local/tmp/test (path inode=1)")


def test_valid_deleted_file_dev_zero():
    """
    Test /dev/zero/
    """
    assert not restartcheck._valid_deleted_file("/dev/zero/test")
    assert not restartcheck._valid_deleted_file("/dev/zero/test (deleted)")
    assert not restartcheck._valid_deleted_file("/dev/zero/test (path inode=1)")


def test_valid_deleted_file_dev_pts():
    """
    Test /dev/pts/
    """
    assert not restartcheck._valid_deleted_file("/dev/pts/test")
    assert not restartcheck._valid_deleted_file("/dev/pts/test (deleted)")
    assert not restartcheck._valid_deleted_file("/dev/pts/test (path inode=1)")


def test_valid_deleted_file_usr_lib_locale():
    """
    Test /usr/lib/locale/
    """
    assert not restartcheck._valid_deleted_file("/usr/lib/locale/test")
    assert not restartcheck._valid_deleted_file("/usr/lib/locale/test (deleted)")
    assert not restartcheck._valid_deleted_file("/usr/lib/locale/test (path inode=1)")


def test_valid_deleted_file_home():
    """
    Test /home/
    """
    assert not restartcheck._valid_deleted_file("/home/test")
    assert not restartcheck._valid_deleted_file("/home/test (deleted)")
    assert not restartcheck._valid_deleted_file("/home/test (path inode=1)")


def test_valid_deleted_file_icon_theme_cache():
    """
    Test /test.icon-theme.cache
    """
    assert not restartcheck._valid_deleted_file("/dev/test.icon-theme.cache")
    assert not restartcheck._valid_deleted_file("/dev/test.icon-theme.cache (deleted)")
    assert not restartcheck._valid_deleted_file(
        "/dev/test.icon-theme.cache (path inode=1)"
    )


def test_valid_deleted_file_var_cache_fontconfig():
    """
    Test /var/cache/fontconfig/
    """
    assert not restartcheck._valid_deleted_file("/var/cache/fontconfig/test")
    assert not restartcheck._valid_deleted_file("/var/cache/fontconfig/test (deleted)")
    assert not restartcheck._valid_deleted_file(
        "/var/cache/fontconfig/test (path inode=1)"
    )


def test_valid_deleted_file_var_lib_nagios3_spool():
    """
    Test /var/lib/nagios3/spool/
    """
    assert not restartcheck._valid_deleted_file("/var/lib/nagios3/spool/test")
    assert not restartcheck._valid_deleted_file("/var/lib/nagios3/spool/test (deleted)")
    assert not restartcheck._valid_deleted_file(
        "/var/lib/nagios3/spool/test (path inode=1)"
    )


def test_valid_deleted_file_var_lib_nagios3_spool_checkresults():
    """
    Test /var/lib/nagios3/spool/checkresults/
    """
    assert not restartcheck._valid_deleted_file(
        "/var/lib/nagios3/spool/checkresults/test"
    )
    assert not restartcheck._valid_deleted_file(
        "/var/lib/nagios3/spool/checkresults/test (deleted)"
    )
    assert not restartcheck._valid_deleted_file(
        "/var/lib/nagios3/spool/checkresults/test (path inode=1)"
    )


def test_valid_deleted_file_var_lib_postgresql():
    """
    Test /var/lib/postgresql/
    """
    assert not restartcheck._valid_deleted_file("/var/lib/postgresql/test")
    assert not restartcheck._valid_deleted_file("/var/lib/postgresql/test (deleted)")
    assert not restartcheck._valid_deleted_file(
        "/var/lib/postgresql/test (path inode=1)"
    )


def test_valid_deleted_file_var_lib_vdr():
    """
    Test /var/lib/vdr/
    """
    assert not restartcheck._valid_deleted_file("/var/lib/vdr/test")
    assert not restartcheck._valid_deleted_file("/var/lib/vdr/test (deleted)")
    assert not restartcheck._valid_deleted_file("/var/lib/vdr/test (path inode=1)")


def test_valid_deleted_file_aio():
    """
    Test /[aio]/
    """
    assert not restartcheck._valid_deleted_file("/opt/test")
    assert not restartcheck._valid_deleted_file("/opt/test (deleted)")
    assert not restartcheck._valid_deleted_file("/opt/test (path inode=1)")
    assert not restartcheck._valid_deleted_file("/apt/test")
    assert not restartcheck._valid_deleted_file("/apt/test (deleted)")
    assert not restartcheck._valid_deleted_file("/apt/test (path inode=1)")
    assert not restartcheck._valid_deleted_file("/ipt/test")
    assert not restartcheck._valid_deleted_file("/ipt/test (deleted)")
    assert not restartcheck._valid_deleted_file("/ipt/test (path inode=1)")
    assert not restartcheck._valid_deleted_file("/aio/test")
    assert not restartcheck._valid_deleted_file("/aio/test (deleted)")
    assert not restartcheck._valid_deleted_file("/aio/test (path inode=1)")


def test_valid_deleted_file_sysv():
    """
    Test /SYSV/
    """
    assert not restartcheck._valid_deleted_file("/SYSV/test")
    assert not restartcheck._valid_deleted_file("/SYSV/test (deleted)")
    assert not restartcheck._valid_deleted_file("/SYSV/test (path inode=1)")


def test_valid_command():
    """
    test for CVE-2020-28243
    """
    create_file = os.path.join(SALT_CODE_DIR, "created_file")

    patch_kernel = patch(
        "salt.modules.restartcheck._kernel_versions_redhat",
        return_value=["3.10.0-1127.el7.x86_64"],
    )
    services = {
        "NetworkManager": {"ExecMainPID": 123},
        "auditd": {"ExecMainPID": 456},
        "crond": {"ExecMainPID": 789},
    }

    patch_salt = patch.dict(
        restartcheck.__salt__,
        {
            "cmd.run": MagicMock(
                return_value="Linux localhost.localdomain 3.10.0-1127.el7.x86_64"
            ),
            "service.get_running": MagicMock(return_value=list(services.keys())),
            "service.show": MagicMock(side_effect=list(services.values())),
            "pkg.owner": MagicMock(return_value=""),
            "service.available": MagicMock(return_value=True),
        },
    )

    patch_deleted = patch(
        "salt.modules.restartcheck._deleted_files",
        MagicMock(return_value=[(f";touch {create_file};", 123, "/root/ (deleted)")]),
    )

    patch_readlink = patch("os.readlink", return_value=f"/root/;touch {create_file};")

    check_error = True
    if salt.utils.path.which("repoquery"):
        check_error = False

    patch_grains = patch.dict(restartcheck.__grains__, {"os_family": "RedHat"})
    with patch_kernel, patch_salt, patch_deleted, patch_readlink, patch_grains:
        if check_error:
            with pytest.raises(FileNotFoundError):
                restartcheck.restartcheck()
        else:
            ret = restartcheck.restartcheck()
            assert "Found 1 processes using old versions of upgraded files" in ret
        assert not os.path.exists(create_file)


def test_valid_command_b():
    """
    test for CVE-2020-28243
    """
    create_file = os.path.join(SALT_CODE_DIR, "created_file")

    patch_kernel = patch(
        "salt.modules.restartcheck._kernel_versions_redhat",
        return_value=["3.10.0-1127.el7.x86_64"],
    )
    services = {
        "NetworkManager": {"ExecMainPID": 123},
        "auditd": {"ExecMainPID": 456},
        "crond": {"ExecMainPID": 789},
    }

    patch_salt = patch.dict(
        restartcheck.__salt__,
        {
            "cmd.run": MagicMock(
                return_value="Linux localhost.localdomain 3.10.0-1127.el7.x86_64"
            ),
            "service.get_running": MagicMock(return_value=list(services.keys())),
            "service.show": MagicMock(side_effect=list(services.values())),
            "pkg.owner": MagicMock(return_value=""),
            "service.available": MagicMock(return_value=True),
        },
    )

    patch_deleted = patch(
        "salt.modules.restartcheck._deleted_files",
        MagicMock(return_value=[("--admindir tmp dpkg", 123, "/root/ (deleted)")]),
    )

    patch_readlink = patch("os.readlink", return_value="--admindir tmp dpkg")

    popen_mock = MagicMock()
    popen_mock.return_value.stdout.readline.side_effect = ["/usr/bin\n", ""]
    patch_popen = patch("subprocess.Popen", popen_mock)

    patch_grains = patch.dict(restartcheck.__grains__, {"os_family": "RedHat"})
    with (
        patch_kernel
    ), patch_salt, patch_deleted, patch_readlink, patch_grains, patch_popen:
        ret = restartcheck.restartcheck()
        assert "Found 1 processes using old versions of upgraded files" in ret
        popen_mock.assert_called_with(
            ["repoquery", "-l", "--admindir tmp dpkg"], stdout=ANY
        )



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>

    Test cases for salt.modules.webutil
"""

import pytest

import salt.modules.webutil as htpasswd
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {htpasswd: {}}


# 'useradd' function tests: 1


def test_useradd():
    """
    Test if it adds an HTTP user using the htpasswd command
    """
    mock = MagicMock(return_value={"out": "Salt"})
    with patch.dict(htpasswd.__salt__, {"cmd.run_all": mock}), patch(
        "os.path.exists", MagicMock(return_value=True)
    ):
        assert htpasswd.useradd("/etc/httpd/htpasswd", "larry", "badpassword") == {
            "out": "Salt"
        }


# 'userdel' function tests: 2


def test_userdel():
    """
    Test if it delete an HTTP user from the specified htpasswd file.
    """
    mock = MagicMock(return_value="Salt")
    with patch.dict(htpasswd.__salt__, {"cmd.run": mock}), patch(
        "os.path.exists", MagicMock(return_value=True)
    ):
        assert htpasswd.userdel("/etc/httpd/htpasswd", "larry") == ["Salt"]


def test_userdel_missing_htpasswd():
    """
    Test if it returns error when no htpasswd file exists
    """
    with patch("os.path.exists", MagicMock(return_value=False)):
        assert (
            htpasswd.userdel("/etc/httpd/htpasswd", "larry")
            == "Error: The specified htpasswd file does not exist"
        )



import logging
import os

import pytest

import salt.serializers.json as jsonserializer
import salt.serializers.msgpack as msgpackserializer
import salt.serializers.yaml as yamlserializer
import salt.states.file as filestate
from salt.exceptions import CommandExecutionError
from tests.support.mock import MagicMock, patch

log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    return {
        filestate: {
            "__env__": "base",
            "__salt__": {"file.manage_file": False},
            "__serializers__": {
                "yaml.serialize": yamlserializer.serialize,
                "yaml.seserialize": yamlserializer.serialize,
                "json.serialize": jsonserializer.serialize,
                "msgpack.serialize": msgpackserializer.serialize,
            },
            "__opts__": {"test": False, "cachedir": ""},
            "__instance_id__": "",
            "__low__": {},
            "__utils__": {},
        }
    }


# 'absent' function tests: 1
def test_absent():
    """
    Test to make sure that the named file or directory is absent.
    """
    name = "/fake/file.conf"

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    mock_t = MagicMock(return_value=True)
    mock_f = MagicMock(return_value=False)
    mock_file = MagicMock(side_effect=[True, CommandExecutionError])
    mock_tree = MagicMock(side_effect=[True, OSError])

    comt = "Must provide name to file.absent"
    ret.update({"comment": comt, "name": ""})

    with patch.object(os.path, "islink", MagicMock(return_value=False)):
        assert filestate.absent("") == ret

        with patch.object(os.path, "isabs", mock_f):
            comt = f"Specified file {name} is not an absolute path"
            ret.update({"comment": comt, "name": name})
            assert filestate.absent(name) == ret

        with patch.object(os.path, "isabs", mock_t):
            comt = 'Refusing to make "/" absent'
            ret.update({"comment": comt, "name": "/"})
            assert filestate.absent("/") == ret

        with patch.object(os.path, "isfile", mock_t):
            with patch.dict(filestate.__opts__, {"test": True}):
                comt = f"File {name} is set for removal"
                ret.update(
                    {
                        "comment": comt,
                        "name": name,
                        "result": None,
                        "changes": {"removed": "/fake/file.conf"},
                    }
                )
                assert filestate.absent(name) == ret

            with patch.dict(filestate.__opts__, {"test": False}):
                with patch.dict(filestate.__salt__, {"file.remove": mock_file}):
                    comt = f"Removed file {name}"
                    ret.update(
                        {"comment": comt, "result": True, "changes": {"removed": name}}
                    )
                    assert filestate.absent(name) == ret

                    comt = f"Removed file {name}"
                    ret.update({"comment": "", "result": False, "changes": {}})
                    assert filestate.absent(name) == ret

        with patch.object(os.path, "isfile", mock_f):
            with patch.object(os.path, "isdir", mock_t):
                with patch.dict(filestate.__opts__, {"test": True}):
                    comt = f"Directory {name} is set for removal"
                    ret.update(
                        {"comment": comt, "changes": {"removed": name}, "result": None}
                    )
                    assert filestate.absent(name) == ret

                with patch.dict(filestate.__opts__, {"test": False}):
                    with patch.dict(filestate.__salt__, {"file.remove": mock_tree}):
                        comt = f"Removed directory {name}"
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {"removed": name},
                            }
                        )
                        assert filestate.absent(name) == ret

                        comt = f"Failed to remove directory {name}"
                        ret.update({"comment": comt, "result": False, "changes": {}})
                        assert filestate.absent(name) == ret

            with patch.object(os.path, "isdir", mock_f):
                with patch.dict(filestate.__opts__, {"test": True}):
                    comt = f"File {name} is not present"
                    ret.update({"comment": comt, "result": True})
                    assert filestate.absent(name) == ret



"""
    Unit tests for the salt.modules.state module
"""

import pytest

import salt.loader.context
import salt.modules.config as config
import salt.modules.state as state
import salt.payload
import salt.utils.files
import salt.utils.state
from tests.support.mock import MagicMock, mock_open, patch


@pytest.fixture
def configure_loader_modules():
    return {
        state: {
            "__salt__": {
                "config.get": config.get,
                "saltutil.is_running": MagicMock(return_value=[]),
            },
            "__opts__": {"test": True, "cachedir": "/tmp/salt_test_cache"},
        },
        config: {"__opts__": {}},
    }


def test_get_initial_pillar():
    """
    _get_initial_pillar returns pillar data not named context
    """
    ctx = salt.loader.context.LoaderContext()
    pillar_data = {"foo": "bar"}
    named_ctx = ctx.named_context("__pillar__", pillar_data)
    opts = {"__cli": "salt-call", "pillarenv": "base"}
    with patch("salt.modules.state.__pillar__", named_ctx, create=True):
        with patch("salt.modules.state.__opts__", opts, create=True):
            pillar = state._get_initial_pillar(opts)
            assert pillar == pillar_data


def test_check_test_value_is_boolean():
    """
    Ensure that the test value is always returned as a boolean
    """
    with patch.dict(state.__opts__, {"test": True}, create=True):
        assert state._get_test_value() is True
        assert state._get_test_value(True) is True
        assert state._get_test_value(False) is False
        assert state._get_test_value("test") is True
        assert state._get_test_value(123) is True

    with patch.dict(state.__opts__, {"test": False}, create=True):
        assert state._get_test_value() is False


def test_check_queue_queues_job_when_conflict():
    """
    Test that _check_queue serializes the job and returns queued=True when prior states exist.
    """
    my_jid = "20230101000000100000"
    older_jid = "20230101000000000000"
    new_jid = "20230101000000300000"

    kwargs = {
        "__pub_jid": my_jid,
        "__pub_fun": "state.apply",
        "__pub_arg": ["foo"],
        "__pub_tgt": "*",
        "__pub_ret": "",
        "__pub_user": "root",
        "concurrent": False,
    }

    # Mock active jobs containing an older job
    active_jobs = [{"jid": older_jid, "pid": 1234, "fun": "state.apply"}]

    opts = {"cachedir": "/tmp/salt_test_cache"}

    # Mock the lock to do nothing
    mock_lock = MagicMock()
    mock_lock.__enter__ = MagicMock(return_value=None)
    mock_lock.__exit__ = MagicMock(return_value=None)

    with patch.dict(state.__opts__, opts, create=True), patch(
        "salt.modules.state.__salt__",
        {"saltutil.is_running": MagicMock(return_value=active_jobs)},
    ), patch("salt.utils.state.acquire_queue_lock", return_value=mock_lock), patch(
        "os.path.exists", return_value=True
    ), patch(
        "os.listdir", return_value=[]
    ), patch(
        "salt.utils.jid.gen_jid", return_value=new_jid
    ), patch(
        "salt.utils.files.fopen", mock_open()
    ) as mock_file, patch(
        "salt.payload.dump"
    ) as mock_dump, patch(
        "salt.utils.atomicfile.atomic_rename"
    ) as mock_rename:

        ret = state._check_queue(True, kwargs)

        # Should return queued dict
        assert isinstance(ret, dict)
        assert ret["queued"] is True
        assert ret["result"] is True
        assert "Job queued for execution" in ret["comment"]

        # Should have dumped payload
        assert mock_dump.called
        args, _ = mock_dump.call_args
        payload = args[0]
        assert payload["jid"] == new_jid
        assert payload["fun"] == "state.apply"


def test_check_queue_proceeds_when_no_conflict():
    """
    Test that _check_queue returns None (proceeds) when no prior states exist.
    """
    my_jid = "20230101000000100000"

    kwargs = {"__pub_jid": my_jid, "concurrent": False}

    # Mock active jobs - EMPTY
    active_jobs = []

    opts = {"cachedir": "/tmp/salt_test_cache"}

    mock_lock = MagicMock()
    mock_lock.__enter__ = MagicMock(return_value=None)
    mock_lock.__exit__ = MagicMock(return_value=None)

    with patch.dict(state.__opts__, opts, create=True), patch(
        "salt.modules.state.__salt__",
        {"saltutil.is_running": MagicMock(return_value=active_jobs)},
    ), patch("salt.utils.state.acquire_queue_lock", return_value=mock_lock), patch(
        "os.path.exists", return_value=False
    ):  # No queue dir

        ret = state._check_queue(True, kwargs)

        # Should return None (no conflict)
        assert ret is None


def test_check_queue_detects_queued_file_as_conflict():
    """
    Test that _check_queue sees a file in state_queue as a conflict and queues itself.
    """
    my_jid = "20230101000000200000"
    older_queued_jid = "20230101000000100000"

    kwargs = {"__pub_jid": my_jid, "__pub_fun": "state.apply", "concurrent": False}

    # Mock active jobs - EMPTY (simulating gap where job is only on disk)
    active_jobs = []

    opts = {"cachedir": "/tmp/salt_test_cache"}

    mock_lock = MagicMock()
    mock_lock.__enter__ = MagicMock(return_value=None)
    mock_lock.__exit__ = MagicMock(return_value=None)

    # Mock os.listdir to return an older queued file
    queued_filename = f"queued_123456_{older_queued_jid}.p"

    with patch.dict(state.__opts__, opts, create=True), patch(
        "salt.modules.state.__salt__",
        {"saltutil.is_running": MagicMock(return_value=active_jobs)},
    ), patch("salt.utils.state.acquire_queue_lock", return_value=mock_lock), patch(
        "os.path.exists", return_value=True
    ), patch(
        "os.listdir", return_value=[queued_filename]
    ), patch(
        "salt.utils.files.fopen", mock_open()
    ) as mock_file, patch(
        "salt.payload.dump"
    ) as mock_dump, patch(
        "salt.utils.atomicfile.atomic_rename"
    ) as mock_rename:

        ret = state._check_queue(True, kwargs)

        # Should return queued dict because it saw the older queued file
        assert isinstance(ret, dict)
        assert ret["queued"] is True
        assert ret["result"] is True

        # Should have dumped payload
        assert mock_dump.called



from collections import OrderedDict

import pytest

from salt.pillar.git_pillar import ext_pillar
from salt.utils.immutabletypes import ImmutableDict, ImmutableList
from tests.support.mock import patch

pytestmark = [
    pytest.mark.slow_test,
]


try:
    import git  # pylint: disable=unused-import

    HAS_GITPYTHON = True
except ImportError:
    HAS_GITPYTHON = False


try:
    import pygit2  # pylint: disable=unused-import

    HAS_PYGIT2 = True
except ImportError:
    HAS_PYGIT2 = False


skipif_no_gitpython = pytest.mark.skipif(not HAS_GITPYTHON, reason="Missing gitpython")
skipif_no_pygit2 = pytest.mark.skipif(not HAS_PYGIT2, reason="Missing pygit2")


@pytest.fixture
def git_pillar_opts(salt_master, tmp_path):
    opts = dict(salt_master.config)
    opts["cachedir"] = str(tmp_path)
    for key, item in opts.items():
        if isinstance(item, ImmutableDict):
            opts[key] = dict(item)
        elif isinstance(item, ImmutableList):
            opts[key] = list(item)
    return opts


@pytest.fixture
def gitpython_pillar_opts(git_pillar_opts):
    git_pillar_opts["verified_git_pillar_provider"] = "gitpython"
    return git_pillar_opts


@pytest.fixture
def pygit2_pillar_opts(git_pillar_opts):
    git_pillar_opts["verified_git_pillar_provider"] = "pygit2"
    return git_pillar_opts


def _get_ext_pillar(minion, pillar_opts, grains, *repos):
    with patch("salt.pillar.git_pillar.__opts__", pillar_opts, create=True):
        with patch("salt.pillar.git_pillar.__grains__", grains, create=True):
            return ext_pillar(minion, None, *repos)


def _test_simple(pillar_opts, grains):
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        "https://github.com/saltstack/salt-test-pillar-gitfs.git",
    )
    assert data == {"key": "value"}


@skipif_no_gitpython
def test_gitpython_simple(gitpython_pillar_opts, grains):
    _test_simple(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_simple(pygit2_pillar_opts, grains):
    _test_simple(pygit2_pillar_opts, grains)


def _test_missing_env(pillar_opts, grains):
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        {
            "https://github.com/saltstack/salt-test-pillar-gitfs.git": [
                {"env": "misssing"}
            ]
        },
    )
    assert data == {}


@skipif_no_gitpython
def test_gitpython_missing_env(gitpython_pillar_opts, grains):
    _test_missing_env(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_missing_env(pygit2_pillar_opts, grains):
    _test_missing_env(pygit2_pillar_opts, grains)


def _test_env(pillar_opts, grains):
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        {
            "other https://github.com/saltstack/salt-test-pillar-gitfs-2.git": [
                {"env": "other_env"}
            ]
        },
    )
    assert data == {"other": "env"}


@skipif_no_gitpython
def test_gitpython_env(gitpython_pillar_opts, grains):
    _test_env(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_env(pygit2_pillar_opts, grains):
    _test_env(pygit2_pillar_opts, grains)


def _test_branch(pillar_opts, grains):
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        "branch https://github.com/saltstack/salt-test-pillar-gitfs.git",
    )
    assert data == {"key": "data"}


@skipif_no_gitpython
def test_gitpython_branch(gitpython_pillar_opts, grains):
    _test_branch(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_branch(pygit2_pillar_opts, grains):
    _test_branch(pygit2_pillar_opts, grains)


def _test_simple_dynamic(pillar_opts, grains):
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        "__env__ https://github.com/saltstack/salt-test-pillar-gitfs.git",
    )
    assert data == {"key": "value"}


@skipif_no_gitpython
def test_gitpython_simple_dynamic(gitpython_pillar_opts, grains):
    _test_simple_dynamic(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_simple_dynamic(pygit2_pillar_opts, grains):
    _test_simple_dynamic(pygit2_pillar_opts, grains)


def _test_missing_env_dynamic(pillar_opts, grains):
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        {
            "__env__ https://github.com/saltstack/salt-test-pillar-gitfs.git": [
                {"env": "misssing"}
            ]
        },
    )
    assert data == {}


@skipif_no_gitpython
def test_gitpython_missing_env_dynamic(gitpython_pillar_opts, grains):
    _test_missing_env_dynamic(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_missing_env_dynamic(pygit2_pillar_opts, grains):
    _test_missing_env_dynamic(pygit2_pillar_opts, grains)


def _test_pillarenv_dynamic(pillar_opts, grains):
    pillar_opts["pillarenv"] = "branch"
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        "__env__ https://github.com/saltstack/salt-test-pillar-gitfs.git",
    )
    assert data == {"key": "data"}


@skipif_no_gitpython
def test_gitpython_pillarenv_dynamic(gitpython_pillar_opts, grains):
    _test_pillarenv_dynamic(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_pillarenv_dynamic(pygit2_pillar_opts, grains):
    _test_pillarenv_dynamic(pygit2_pillar_opts, grains)


def _test_multiple(pillar_opts, grains):
    pillar_opts["pillarenv"] = "branch"
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        "__env__ https://github.com/saltstack/salt-test-pillar-gitfs.git",
        "other https://github.com/saltstack/salt-test-pillar-gitfs-2.git",
    )
    assert data == {"key": "data"}


@skipif_no_gitpython
def test_gitpython_multiple(gitpython_pillar_opts, grains):
    _test_multiple(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_multiple(pygit2_pillar_opts, grains):
    _test_multiple(pygit2_pillar_opts, grains)


def _test_multiple_2(pillar_opts, grains):
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        "https://github.com/saltstack/salt-test-pillar-gitfs.git",
        "https://github.com/saltstack/salt-test-pillar-gitfs-2.git",
    )
    assert data == {
        "key": "value",
        "key1": "value1",
        "key2": "value2",
        "key4": "value4",
        "data1": "d",
        "data2": "d2",
    }


@skipif_no_gitpython
def test_gitpython_multiple_2(gitpython_pillar_opts, grains):
    _test_multiple_2(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_multiple_2(pygit2_pillar_opts, grains):
    _test_multiple_2(pygit2_pillar_opts, grains)


def _test_multiple_slash_in_branch_name(pillar_opts, grains):
    pillar_opts["pillarenv"] = "doggy/moggy"
    data = _get_ext_pillar(
        "minion",
        pillar_opts,
        grains,
        "__env__ https://github.com/saltstack/salt-test-pillar-gitfs.git",
        "other https://github.com/saltstack/salt-test-pillar-gitfs-2.git",
    )
    assert data == {
        "key": "data",
        "foo": OrderedDict(
            [
                ("animals", OrderedDict([("breed", "seadog")])),
                (
                    "feature/baz",
                    OrderedDict(
                        [("test1", "dog"), ("test2", "kat"), ("test3", "gerbil")]
                    ),
                ),
            ]
        ),
    }


@skipif_no_gitpython
def test_gitpython_multiple_slash_in_branch_name(gitpython_pillar_opts, grains):
    _test_multiple_slash_in_branch_name(gitpython_pillar_opts, grains)


@skipif_no_pygit2
def test_pygit2_multiple_slash_in_branch_name(pygit2_pillar_opts, grains):
    _test_multiple_slash_in_branch_name(pygit2_pillar_opts, grains)



"""
Tests for the spm install utility
"""

import os

import pytest

import salt.utils.files
import salt.utils.path
import salt.utils.yaml
from tests.support.case import SPMCase
from tests.support.helpers import Webserver


@pytest.mark.destructive_test
@pytest.mark.windows_whitelisted
class SPMInstallTest(SPMCase):
    """
    Validate the spm install command
    """

    def setUp(self):
        self.config = self._spm_config()
        self._spm_build_files(self.config)
        self.spm_build_dir = self.config["spm_build_dir"]
        if "http" in self.id():
            # only start the webserver when testing http
            self.webserver = Webserver()
            self.webserver.root = self.spm_build_dir
            self.webserver.start()
            self.addCleanup(self.webserver.stop)
            self.repo_dir = self.config["spm_repos_config"] + ".d"
            self.repo = os.path.join(self.repo_dir, "spm.repo")
            url = {"my_repo": {"url": self.webserver.url("")[:-1]}}

            if not os.path.exists(self.repo_dir):
                os.makedirs(self.repo_dir)

            with salt.utils.files.fopen(self.repo, "w") as fp:
                salt.utils.yaml.safe_dump(url, fp)

    def test_spm_install_http(self):
        """
        test spm install using http repo
        """
        build_spm = self.run_spm("build", self.config, self.formula_dir)
        spm_file = os.path.join(self.spm_build_dir, "apache-201506-2.spm")

        create_repo = self.run_spm("create_repo", self.config, self.spm_build_dir)

        for root, dirs, files in salt.utils.path.os_walk(self.spm_build_dir):
            for fp in files:
                self.webserver.url(fp)

        install = self.run_spm("install", self.config, "apache")

        sls = os.path.join(self.config["formula_path"], "apache", "apache.sls")

        self.assertTrue(os.path.exists(sls))

    @pytest.mark.slow_test
    def test_spm_install_local_dir(self):
        """
        test spm install from local directory
        """
        build_spm = self.run_spm("build", self.config, self.formula_dir)
        spm_file = os.path.join(self.config["spm_build_dir"], "apache-201506-2.spm")

        install = self.run_spm("install", self.config, spm_file)

        sls = os.path.join(self.config["formula_path"], "apache", "apache.sls")

        self.assertTrue(os.path.exists(sls))

    @pytest.mark.slow_test
    def test_spm_install_from_repo(self):
        """
        test spm install from repo
        """
        self._spm_create_update_repo(self.config)
        install = self.run_spm("install", self.config, "apache")

        sls = os.path.join(self.config["formula_path"], "apache", "apache.sls")

        self.assertTrue(os.path.exists(sls))



from textwrap import dedent

import pytest

pytest.importorskip("grp")

import grp

import salt.utils.user


@pytest.fixture(scope="function")
def etc_group(tmp_path):
    etcgrp = tmp_path / "etc" / "group"
    etcgrp.parent.mkdir()
    etcgrp.write_text(
        dedent(
            """games:x:50:
            docker:x:959:debian,salt
            salt:x:1000:"""
        )
    )
    return etcgrp


def test__getgrall(etc_group):
    group_lines = [
        ["games", "x", 50, []],
        ["docker", "x", 959, ["debian", "salt"]],
        ["salt", "x", 1000, []],
    ]
    expected_grall = [grp.struct_group(comps) for comps in group_lines]

    grall = salt.utils.user._getgrall(root=str(etc_group.parent.parent))

    assert grall == expected_grall


def test__getgrall_bad_format(etc_group):
    with etc_group.open("a") as _fp:
        _fp.write("\n# some comment here\n")

    with pytest.raises(IndexError):
        salt.utils.user._getgrall(root=str(etc_group.parent.parent))



"""
    Test win_chcp
"""

import pytest

from salt.exceptions import CodePageError
from salt.utils import win_chcp
from tests.support.unit import TestCase

pytestmark = [
    pytest.mark.skip_unless_on_windows,
]


class CHCPTest(TestCase):
    """
    Test case for salt.utils.win_chcp
    """

    @classmethod
    def setUpClass(cls):
        # Stores the initial code page for _reset_code_page()
        # Intentionally does not catch any exception, to find out which that would be.
        # That exception would then be added to cmdmod.py
        cls._chcp_code = win_chcp.get_codepage_id()

    @classmethod
    def tearDownClass(cls):
        cls._chcp_code = None

    def setUp(self):
        win_chcp.set_codepage_id(self._chcp_code)

    def tearDown(self):
        win_chcp.set_codepage_id(self._chcp_code)

    def test_get_and_set_code_page(self):
        for page in (20424, "20866", 437, 65001, "437"):
            self.assertEqual(win_chcp.set_codepage_id(page), int(page))
            self.assertEqual(win_chcp.get_codepage_id(), int(page))

    def test_bad_page_code(self):
        with win_chcp.chcp(437):
            self.assertEqual(win_chcp.get_codepage_id(), 437)

            bad_codes = ("0", "bad code", 1234, -34, "437 dogs", "(*&^(*^%&$%&")

            for page in bad_codes:
                self.assertEqual(win_chcp.set_codepage_id(page), -1)
                self.assertEqual(win_chcp.get_codepage_id(), 437)

            for page in bad_codes:
                self.assertRaises(CodePageError, win_chcp.set_codepage_id, page, True)



import pytest

import salt.serializers.json as jsonserializer
import salt.serializers.msgpack as msgpackserializer
import salt.serializers.yaml as yamlserializer
import salt.states.file as filestate
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {
        filestate: {
            "__env__": "base",
            "__salt__": {"file.manage_file": False},
            "__serializers__": {
                "yaml.serialize": yamlserializer.serialize,
                "yaml.seserialize": yamlserializer.serialize,
                "json.serialize": jsonserializer.serialize,
                "msgpack.serialize": msgpackserializer.serialize,
            },
            "__opts__": {"test": False, "cachedir": ""},
            "__instance_id__": "",
            "__low__": {},
            "__utils__": {},
        }
    }


def test_file_serialize_tmp_dir_system_temp(tmp_path):
    tmp_file = tmp_path / "tmp.txt"
    mock_mkstemp = MagicMock()
    with patch("salt.utils.files.mkstemp", mock_mkstemp), patch.dict(
        filestate.__salt__,
        {
            "cmd.run_all": MagicMock(return_value={"retcode": 0}),
            "file.file_exists": MagicMock(return_value=False),
            "file.manage_file": MagicMock(),
        },
    ):
        filestate.serialize(str(tmp_file), dataset={"wollo": "herld"}, check_cmd="true")
        mock_mkstemp.assert_called_with(suffix="", dir=None)



"""
    :codeauthor: :email:`David Murphy <david-dm.murphy@broadcom.com`
"""

import shutil

import pytest

import salt.grains.extra
from tests.support.mock import patch

pytestmark = [
    pytest.mark.skip_unless_on_linux(reason="Only supported on Linux family"),
]


@pytest.mark.parametrize(
    "setting_secure, extra_file, expected_enabled",
    (
        (True, False, True),
        (True, True, False),
        (False, False, False),
        (False, True, False),
    ),
)
def test_secure_boot_efivars(tmp_path, setting_secure, extra_file, expected_enabled):
    secure_boot_path = tmp_path / "secure-boot"
    secure_boot_path_vars = secure_boot_path / "efivars"
    secure_boot_path_vars.mkdir(parents=True, exist_ok=True)
    secure_boot_filepath = secure_boot_path_vars / "SecureBoot-dog"

    if setting_secure:
        secure_boot_filepath.write_bytes(b"\x06\x00\x00\x00\x01")
    else:
        secure_boot_filepath.write_bytes(b"\x06\x00\x00\x00\x00")

    if extra_file:
        secure_boot_filepath2 = secure_boot_path_vars / "SecureBoot-kat"
        if setting_secure:
            secure_boot_filepath2.write_bytes(b"\x06\x00\x00\x00\x01")
        else:
            secure_boot_filepath2.write_bytes(b"\x06\x00\x00\x00\x00")

    with patch(
        "salt.grains.extra.get_secure_boot_path", return_value=secure_boot_path_vars
    ):
        grains = salt.grains.extra.uefi()
        expected = {"efi": True, "efi-secure-boot": expected_enabled}
        assert grains == expected

    shutil.rmtree(secure_boot_path)


@pytest.mark.parametrize(
    "setting_secure, extra_file, expected_enabled",
    (
        (True, False, True),
        (True, True, False),
        (False, False, False),
        (False, True, False),
    ),
)
def test_secure_boot_vars(tmp_path, setting_secure, extra_file, expected_enabled):
    secure_boot_path = tmp_path / "secure-boot"
    secure_boot_path_vars = secure_boot_path / "vars" / "SecureBoot-dog"
    secure_boot_path_vars1 = secure_boot_path_vars / "SecureBoot-dog"
    secure_boot_path_vars1.mkdir(parents=True, exist_ok=True)
    secure_boot_filepath = secure_boot_path_vars1 / "data"

    if setting_secure:
        secure_boot_filepath.write_bytes(b"\x06\x00\x00\x00\x01")
    else:
        secure_boot_filepath.write_bytes(b"\x06\x00\x00\x00\x00")

    if extra_file:
        secure_boot_path_vars2 = secure_boot_path_vars / "SecureBoot-kat"
        secure_boot_path_vars2.mkdir(parents=True, exist_ok=True)
        secure_boot_filepath2 = secure_boot_path_vars2 / "data"
        if setting_secure:
            secure_boot_filepath2.write_bytes(b"\x06\x00\x00\x00\x01")
        else:
            secure_boot_filepath2.write_bytes(b"\x06\x00\x00\x00\x00")

    with patch(
        "salt.grains.extra.get_secure_boot_path", return_value=secure_boot_path_vars
    ):
        grains = salt.grains.extra.uefi()
        expected = {"efi": True, "efi-secure-boot": expected_enabled}
        assert grains == expected

    shutil.rmtree(secure_boot_path)


@pytest.mark.parametrize(
    "setting_secure, expected_enabled",
    (
        (True, True),
        (False, False),
        (False, False),
        (False, False),
    ),
)
def test_secure_boot_efivars_and_vars(tmp_path, setting_secure, expected_enabled):
    secure_boot_path = tmp_path / "secure-boot"
    secure_boot_path_vars = secure_boot_path / "efivars"
    secure_boot_path_vars.mkdir(parents=True, exist_ok=True)
    secure_boot_filepath = secure_boot_path_vars / "SecureBoot-dog"

    secure_boot_path_vars2 = secure_boot_path / "vars" / "SecureBoot-kat"
    secure_boot_path_vars2.mkdir(parents=True, exist_ok=True)
    secure_boot_filepath2 = secure_boot_path_vars2 / "data"

    if setting_secure:
        # efivars True, vars / data False
        secure_boot_filepath.write_bytes(b"\x06\x00\x00\x00\x01")
        secure_boot_filepath2.write_bytes(b"\x06\x00\x00\x00\x00")
    else:
        # efivars false, vars / data True
        secure_boot_filepath.write_bytes(b"\x06\x00\x00\x00\x00")
        secure_boot_filepath2.write_bytes(b"\x06\x00\x00\x00\x01")

    with patch(
        "salt.grains.extra.get_secure_boot_path", return_value=secure_boot_path_vars
    ):
        grains = salt.grains.extra.uefi()
        expected = {"efi": True, "efi-secure-boot": expected_enabled}
        assert grains == expected

    shutil.rmtree(secure_boot_path)



import logging
import os
import subprocess
import time

import pytest

import salt.utils.files

pytestmark = [
    pytest.mark.skip_unless_on_linux(reason="Only supported on Linux family"),
]

log = logging.getLogger(__name__)


@pytest.fixture
def revert_permissions(call_cli):
    """
    Fixture to revert permissions and configuration changes made during the test.

    This is critical because upgrade tests run in a shared environment (container)
    where the package is upgraded but NOT uninstalled between tests (to allow inspection).
    If a test modifies global configuration (like /etc/salt/master user) or file ownership,
    and fails before cleaning up, subsequent tests (including integration tests) will
    run in a dirty environment and likely fail.

    This fixture ensures that:
    1. Services are stopped to release locks/files.
    2. User configuration in /etc/salt/{master,minion} is reverted to defaults (root).
    3. File ownership is restored to root:root.
    4. Services are restarted to apply clean configuration.

    NOTE: We use subprocess.run instead of call_cli.run for cleanup because if the
    test fails while the minion is configured to run as 'salt', salt-call might also
    drop privileges and fail to perform root-level cleanup operations. Using subprocess
    ensures we run as root (the user running the tests).
    """
    yield
    log.info("Reverting permissions and configuration to defaults")

    # Stop services first
    test_list = ["salt-api", "salt-minion", "salt-master"]
    for test_item in test_list:
        subprocess.run(["systemctl", "stop", test_item], check=False)

    # Revert config files - comment out 'user:' lines to default to root
    # Use sed directly to avoid salt-call permission issues
    for config_file in ["/etc/salt/master", "/etc/salt/minion"]:
        if os.path.exists(config_file):
            subprocess.run(["sed", "-i", "s/^user:/#user:/g", config_file], check=False)
            # Ensure root user is set explicitly if needed, or just rely on commenting out
            # Appending 'user: root' might be safer if previous appends are still there
            with salt.utils.files.fopen(config_file, "a") as f:
                f.write("\nuser: root\n")

    # Restore ownership of runtime directories to root:root
    # This is a broad cleanup to ensure no files are left owned by 'horse' or 'donkey'
    dirs = [
        "/etc/salt/pki",
        "/var/cache/salt",
        "/var/log/salt",
        "/var/run/salt",
        "/opt/saltstack/salt",
    ]
    for d in dirs:
        if os.path.exists(d):
            subprocess.run(["chown", "-R", "root:root", d], check=False)

    # Also fix minion_id which might have been created/owned by non-root user
    if os.path.exists("/etc/salt/minion_id"):
        subprocess.run(["chown", "root:root", "/etc/salt/minion_id"], check=False)

    # Also fix /etc/salt/minion.d/_schedule.conf which might have been created/owned by non-root user
    if os.path.exists("/etc/salt/minion.d/_schedule.conf"):
        subprocess.run(
            ["chown", "root:root", "/etc/salt/minion.d/_schedule.conf"], check=False
        )

    # Restart services
    for test_item in test_list:
        subprocess.run(["systemctl", "start", test_item], check=False)

    time.sleep(10)  # Allow time for restart


def test_salt_ownership_permission(
    call_cli, install_salt_systemd, salt_systemd_setup, revert_permissions
):
    """
    Test upgrade of Salt packages preserve existing ownership
    """
    if not install_salt_systemd.upgrade:
        pytest.skip("Not testing an upgrade, do not run")

    test_list = ["salt-api", "salt-minion", "salt-master"]

    # ensure services are started
    for test_item in test_list:
        test_cmd = f"systemctl restart {test_item}"
        try:
            ret = call_cli.run("--local", "cmd.run", test_cmd)
            assert ret.returncode == 0
        except (OSError, AssertionError) as e:
            # Skip if systemd operations fail due to environment issues
            pytest.skip(f"Systemd service management failed for {test_item}: {e}")

    time.sleep(10)  # allow some time for restart

    # test ownership for Minion, Master and Api
    for test_item in test_list:
        test_cmd = f"ls -dl /run/{test_item}.pid"
        try:
            ret = call_cli.run("--local", "cmd.run", test_cmd)
            assert ret.returncode == 0

            test_user = ret.stdout.strip().split()[4]
            test_group = ret.stdout.strip().split()[5]

            if test_item == "salt-minion":
                assert test_user == "root"
                assert test_group == "root"
            else:
                assert test_user == "salt"
                assert test_group == "salt"
        except (OSError, AssertionError, IndexError) as e:
            # Skip if file operations or parsing fail due to environment issues
            pytest.skip(f"File ownership check failed for {test_item}: {e}")

    # create master user, and minion user, change conf, restart and test ownership
    test_master_user = "horse"
    test_minion_user = "donkey"

    ret = call_cli.run("--local", "user.list_users")
    user_list = ret.stdout.strip().split(":")[1]

    if test_master_user not in user_list:
        ret = call_cli.run("--local", "user.add", f"{test_master_user}", usergroup=True)

    if test_minion_user not in user_list:
        ret = call_cli.run("--local", "user.add", f"{test_minion_user}", usergroup=True)

    ret = call_cli.run("--local", "file.comment_line", "/etc/salt/master", "^user:")
    assert ret.returncode == 0

    ret = call_cli.run("--local", "file.comment_line", "/etc/salt/minion", "^user:")
    assert ret.returncode == 0

    test_string = f"\nuser: {test_master_user}\n"
    ret = call_cli.run("--local", "file.append", "/etc/salt/master", test_string)

    test_string = f"\nuser: {test_minion_user}\n"
    ret = call_cli.run("--local", "file.append", "/etc/salt/minion", test_string)

    # Check if the current salt-call version supports --priv option
    # We do this before the upgrade since we're still on the old version
    help_ret = call_cli.run("--help")
    supports_priv = "--priv" in help_ret.stdout

    # restart and check ownership is correct
    # Use --priv=root if supported, otherwise run without it
    test_list = ["salt-api", "salt-minion", "salt-master"]
    for test_item in test_list:
        test_cmd = f"systemctl restart {test_item}"
        if supports_priv:
            ret = call_cli.run("--local", "--priv=root", "cmd.run", test_cmd)
        else:
            ret = call_cli.run("--local", "cmd.run", test_cmd)

    time.sleep(10)  # allow some time for restart

    # restart and check ownership is correct
    try:
        test_list = ["salt-api", "salt-minion", "salt-master"]
        for test_item in test_list:
            test_cmd = f"systemctl restart {test_item}"
            ret = call_cli.run("--local", "cmd.run", test_cmd)
            assert ret.returncode == 0

        time.sleep(10)  # allow some time for restart

        # test ownership for Minion, Master and Api - horse and donkey
        test_list = ["salt-api", "salt-minion", "salt-master"]
        for test_item in test_list:
            test_cmd = f"ls -dl /run/{test_item}.pid"
            ret = call_cli.run("--local", "cmd.run", test_cmd)
            assert ret.returncode == 0

            test_user = ret.stdout.strip().split()[4]
            test_group = ret.stdout.strip().split()[5]

            if test_item == "salt-minion":
                assert test_user == f"{test_minion_user}"
                assert test_group == f"{test_minion_user}"
            else:
                assert test_user == f"{test_master_user}"
                assert test_group == f"{test_master_user}"
    except (OSError, AssertionError, IndexError) as e:
        # Skip if service restart or final ownership check fails due to environment issues
        pytest.skip(f"Service restart or final ownership check failed: {e}")

    # Upgrade Salt (inc. minion, master, etc.) from previous version and test
    # pylint: disable=pointless-statement
    install_salt_systemd.install(upgrade=True)
    time.sleep(60)  # give it some time

    # test ownership for Minion, Master and Api
    test_list = ["salt-api", "salt-minion", "salt-master"]
    for test_item in test_list:
        test_cmd = f"ls -dl /run/{test_item}.pid"
        ret = call_cli.run("--local", "--priv=root", "cmd.run", test_cmd)
        assert ret.returncode == 0

        test_user = ret.stdout.strip().split()[4]
        test_group = ret.stdout.strip().split()[5]

        if test_item == "salt-minion":
            assert test_user == f"{test_minion_user}"
            assert test_group == f"{test_minion_user}"
        else:
            assert test_user == f"{test_master_user}"
            assert test_group == f"{test_master_user}"

    # Cleanup is now handled by the revert_permissions fixture



import hashlib
import os
import zipfile

import pytest
from saltfactories.utils.functional import MultiStateResult

import salt.utils.files
import salt.utils.stringutils
from tests.support.runtests import RUNTIME_VARS


def test_get_file_from_env_in_top_match(salt_cli, salt_sub_minion):
    tgt = os.path.join(RUNTIME_VARS.TMP, "prod-cheese-file")
    try:
        ret = salt_cli.run("state.highstate", minion_tgt=salt_sub_minion.id)
        assert ret.returncode == 0
        assert os.path.isfile(tgt)
        with salt.utils.files.fopen(tgt, "r") as cheese:
            data = salt.utils.stringutils.to_unicode(cheese.read())
        assert "Gromit" in data
        assert "Comte" in data
    finally:
        if os.path.exists(tgt):
            os.unlink(tgt)


def test_issue_56131(salt_minion, base_env_state_tree_root_dir, tmp_path):
    """
    archive.extracted fails if setting an unless clause and pip is not installed.
    """
    zipfile_path = base_env_state_tree_root_dir / "issue-56131.zip"
    with zipfile.ZipFile(
        str(zipfile_path), "w", compression=zipfile.ZIP_DEFLATED
    ) as myzip:
        myzip.writestr("issue-56131.txt", "issue-56131")
    zipfile_hash = hashlib.sha256(zipfile_path.read_bytes()).hexdigest()
    sls_contents = """
    {}:
      archive.extracted:
        - source: salt://issue-56131.zip
        - source_hash: sha256={}
        - archive_format: zip
        - enforce_toplevel: False
        - unless:
          - echo hello; exit 1
    """.format(
        tmp_path, zipfile_hash
    )
    pip_path_dir = tmp_path / "fakepip"
    pip_path_dir.mkdir()
    pip_path = pip_path_dir / "pip.py"
    pip_path.write_text('raise ImportError("No module named pip")')

    environ = os.environ.copy()
    pythonpath = os.environ.get("PYTHONPATH")
    if pythonpath is None:
        pythonpath = str(pip_path_dir)
    else:
        pythonpath = os.pathsep.join([str(pip_path_dir)] + pythonpath.split(os.pathsep))
    environ["PYTHONPATH"] = pythonpath
    salt_call_cli = salt_minion.salt_call_cli(environ=environ)
    extract_path = tmp_path / "issue-56131.txt"
    try:
        assert extract_path.exists() is False
        with pytest.helpers.temp_file(
            "issue-56131.sls", sls_contents, base_env_state_tree_root_dir
        ):
            ret = salt_call_cli.run("state.sls", "issue-56131")
            assert ret.returncode == 0
            for state_return in MultiStateResult(ret.data):
                assert state_return.result is True
            assert extract_path.exists()
    finally:
        zipfile_path.unlink()


@pytest.mark.skip_unless_on_windows(
    reason="Tested in tests/pytests/functional/modules/state/test_state.py"
)
def test_pydsl(salt_call_cli, base_env_state_tree_root_dir, tmp_path):
    """
    Test the basics of the pydsl
    """
    testfile = tmp_path / "testfile"
    sls_contents = """
    #!pydsl

    state("{}").file("touch")
    """.format(
        testfile
    )
    with pytest.helpers.temp_file(
        "pydsl.sls", sls_contents, base_env_state_tree_root_dir
    ):
        ret = salt_call_cli.run("state.sls", "pydsl")
        assert ret.returncode == 0
        assert ret.data
        for state_return in MultiStateResult(ret.data):
            assert state_return.result is True
        assert testfile.exists()


@pytest.mark.skip_unless_on_windows(
    reason="Tested in tests/pytests/functional/modules/state/test_state.py"
)
def test_state_sls_unicode_characters(salt_call_cli, base_env_state_tree_root_dir):
    """
    test state.sls when state file contains non-ascii characters
    """
    sls_contents = """
    echo1:
      cmd.run:
        - name: "echo 'This is Æ test!'"
    """
    with pytest.helpers.temp_file(
        "issue-46672.sls", sls_contents, base_env_state_tree_root_dir
    ):
        ret = salt_call_cli.run("state.sls", "issue-46672")
        assert ret.returncode == 0
        assert ret.data
        expected = "cmd_|-echo1_|-echo 'This is Æ test!'_|-run"
        assert expected in ret.data


@pytest.mark.skip_on_windows(reason="umask is a no-op on Windows")
@pytest.mark.parametrize("umask", (22, "022"))
def test_umask_022(salt_call_cli, umask):
    """
    Should produce a file with mode 644
    """
    with pytest.helpers.temp_file() as name:
        name.unlink()
        salt_call_cli.run("state.single", fun="file.touch", name=str(name), umask=umask)
        assert oct(name.stat().st_mode)[-3:] == "644"


@pytest.mark.skip_on_windows(reason="umask is a no-op on Windows")
@pytest.mark.parametrize("umask", (27, "027"))
def test_umask_027(salt_call_cli, umask):
    """
    Should produce a file with mode 640
    """
    with pytest.helpers.temp_file() as name:
        name.unlink()
        salt_call_cli.run("state.single", fun="file.touch", name=str(name), umask=umask)
        assert oct(name.stat().st_mode)[-3:] == "640"


@pytest.mark.skip_on_windows(reason="umask is a no-op on Windows")
@pytest.mark.parametrize("umask", (999, "999", "foo"))
def test_umask_invalid(salt_call_cli, umask):
    """
    Test invalid umask values
    """
    with pytest.helpers.temp_file() as name:
        name.unlink()
        ret = salt_call_cli.run(
            "state.single", fun="file.touch", name=str(name), umask=umask
        )
        assert ret.data == [f"Invalid umask: {umask}"]



import pytest

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.core_test,
    pytest.mark.timeout_unless_on_windows(240),
]


def test_unless_req(state):
    ret = state.single(fun="test.succeed_with_changes", name="unless test", unless=[{}])
    assert ret.result is True
    assert ret.comment == "Success!"

    ret = state.single(
        fun="test.fail_with_changes", name="unless test", unless=[{"fun": "test.true"}]
    )
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "unless condition is true"

    ret = state.single(
        fun="test.fail_with_changes", name="unless test", unless=[{"fun": "test.false"}]
    )
    assert ret.result is False
    assert ret.changes
    assert ret.comment == "Failure!"

    ret = state.single(
        fun="test.succeed_without_changes",
        name="unless test",
        unless=[{"fun": "test.false"}],
    )
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "Success!"


def test_unless_req_retcode(state):
    ret = state.single(
        fun="test.succeed_with_changes",
        name="unless test",
        unless=[{"fun": "test.retcode"}],
    )
    assert ret.result is True
    assert ret.changes
    assert ret.comment == "Success!"

    ret = state.single(
        fun="test.succeed_with_changes",
        name="unless test",
        unless=[{"fun": "test.retcode", "code": 0}],
    )
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "unless condition is true"



"""
    tests.unit.utils.aws_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Test the salt aws functions
"""

import io
import os
import time
from datetime import timedelta

import pytest
import requests
from pytest_timeout import DEFAULT_METHOD

import salt.utils.aws as aws
import salt.utils.timeutil
from tests.support.helpers import patched_environ
from tests.support.mock import MagicMock, patch

pytestmark = [
    # Skip testing on windows since it does not support signal.SIGALRM
    # which is what the timeout marker is using by default.
    pytest.mark.skip_on_windows,
    pytest.mark.timeout(60, method=DEFAULT_METHOD, func_only=True),
]


@pytest.fixture(autouse=True)
def _cleanup():
    # Make sure this cache is clear before each test
    aws.__AssumeCache__.clear()
    # Remove any AWS_ prefixed environment variables
    with patched_environ(
        __cleanup__=[k for k in os.environ if k.startswith("AWS_")],
    ):
        yield


def test_get_metadata_imdsv1():
    response = requests.Response()
    response.status_code = 200
    response.reason = "OK"
    response.raw = io.BytesIO(b"""test""")
    with patch("requests.get", return_value=response):
        result = aws.get_metadata("/")
        assert result.text == "test"


def test_get_metadata_imdsv2():
    mock_token = "abc123"

    def handle_get_mock(_, **args):
        response = requests.Response()
        if (
            "X-aws-ec2-metadata-token" in args["headers"]
            and args["headers"]["X-aws-ec2-metadata-token"] == mock_token
        ):
            response.status_code = 200
            response.reason = "OK"
            response.raw = io.BytesIO(b"""test""")
        else:
            response.status_code = 401
            response.reason = "Unauthorized"
        return response

    put_response = requests.Response()
    put_response.status_code = 200
    put_response.reason = "OK"
    put_response.raw = io.BytesIO(mock_token.encode("utf-8"))

    with patch("requests.get", MagicMock(side_effect=handle_get_mock)), patch(
        "requests.put", return_value=put_response
    ):
        result = aws.get_metadata("/")
        assert result.text == "test"


def test_assumed_creds_not_updating_dictionary_while_iterating():
    mock_cache = {
        "expired": {
            "Expiration": time.mktime(salt.utils.timeutil.utcnow().timetuple()),
        },
        "not_expired_1": {
            "Expiration": time.mktime(
                (salt.utils.timeutil.utcnow() + timedelta(days=1)).timetuple()
            ),
            "AccessKeyId": "mock_AccessKeyId",
            "SecretAccessKey": "mock_SecretAccessKey",
            "SessionToken": "mock_SessionToken",
        },
        "not_expired_2": {
            "Expiration": time.mktime(
                (salt.utils.timeutil.utcnow() + timedelta(seconds=300)).timetuple()
            ),
        },
    }
    with patch.dict(aws.__AssumeCache__, mock_cache):
        ret = aws.assumed_creds({}, "not_expired_1")
        assert "expired" not in aws.__AssumeCache__
        assert ret == ("mock_AccessKeyId", "mock_SecretAccessKey", "mock_SessionToken")


def test_assumed_creds_deletes_expired_key():
    mock_cache = {
        "expired": {
            "Expiration": time.mktime(salt.utils.timeutil.utcnow().timetuple()),
        },
        "not_expired_1": {
            "Expiration": time.mktime(
                (salt.utils.timeutil.utcnow() + timedelta(days=1)).timetuple()
            ),
            "AccessKeyId": "mock_AccessKeyId",
            "SecretAccessKey": "mock_SecretAccessKey",
            "SessionToken": "mock_SessionToken",
        },
        "not_expired_2": {
            "Expiration": time.mktime(
                (salt.utils.timeutil.utcnow() + timedelta(seconds=300)).timetuple()
            ),
        },
    }
    creds_dict = {
        "AccessKeyId": "mock_AccessKeyId",
        "SecretAccessKey": "mock_SecretAccessKey",
        "SessionToken": "mock_SessionToken",
    }
    response_mock = MagicMock()
    response_mock.status_code = 200
    response_mock.json.return_value = {
        "AssumeRoleResponse": {
            "AssumeRoleResult": {
                "Credentials": creds_dict,
            },
        },
    }
    with patch.dict(aws.__AssumeCache__, mock_cache):
        with patch.object(aws, "sig4", return_value=({}, "fakeurl.com")):
            with patch("requests.request", return_value=response_mock):
                ret = aws.assumed_creds({}, "expired")
                assert "expired" in aws.__AssumeCache__
                assert aws.__AssumeCache__["expired"] == creds_dict
                assert ret == (
                    "mock_AccessKeyId",
                    "mock_SecretAccessKey",
                    "mock_SessionToken",
                )


def test_creds_with_role_arn_should_always_call_assumed_creds():
    role_arn = "arn:aws:iam::111111111111:role/my-role-to-assume"

    access_key_id = "mock_AccessKeyId"
    secret_access_key = "mock_SecretAccessKey"
    token = "mock_Token"
    expiration = (salt.utils.timeutil.utcnow() + timedelta(seconds=900)).strftime(
        "%Y-%m-%dT%H:%M:%SZ"
    )

    patch_expiration = patch("salt.utils.aws.__Expiration__", new=expiration)

    def handle_get_metadata_mock(path, **args):
        response_metadata = MagicMock()
        response_metadata.status_code = 200
        if path == "meta-data/iam/security-credentials/":
            response_metadata.text = "Role"
        else:
            response_metadata.json.return_value = {
                "AccessKeyId": access_key_id,
                "SecretAccessKey": secret_access_key,
                "Token": token,
                "Expiration": expiration,
            }
        return response_metadata

    patch_get_metadata = patch.object(
        aws, "get_metadata", side_effect=handle_get_metadata_mock
    )

    assumed_access_key_id = "mock_assumed_AccessKeyId"
    assumed_secret_access_key = "mock_assumed_SecretAccessKey"
    assumed_session_token = "mock_assumed_SessionToken"
    assumed_creds_ret = (
        assumed_access_key_id,
        assumed_secret_access_key,
        assumed_session_token,
    )

    patch_assumed_creds = patch.object(
        aws, "assumed_creds", return_value=assumed_creds_ret
    )

    # test for the first call, with __Expiration__ = "" (default)
    with patch_get_metadata as mock_get_metadata:
        with patch_assumed_creds:
            result = aws.creds(
                {"id": aws.IROLE_CODE, "key": aws.IROLE_CODE, "role_arn": role_arn}
            )
            assert mock_get_metadata.call_count == 2
            assert result == assumed_creds_ret

    # test for the second call, with valid __Expiration__
    with patch_get_metadata as mock_get_metadata:
        with patch_expiration, patch_assumed_creds:
            result = aws.creds(
                {"id": aws.IROLE_CODE, "key": aws.IROLE_CODE, "role_arn": role_arn}
            )
            assert mock_get_metadata.call_count == 0
            assert result == assumed_creds_ret



"""
Integration tests for the rabbitmq_user states
"""

import logging

import pytest

import salt.modules.rabbitmq as rabbitmq
import salt.states.rabbitmq_upstream as rabbitmq_upstream
from salt.utils.versions import Version

log = logging.getLogger(__name__)

docker = pytest.importorskip("docker")

pytestmark = [
    pytest.mark.slow_test,
    pytest.mark.skip_if_binaries_missing(
        "docker", "dockerd", reason="Docker not installed"
    ),
    pytest.mark.skipif(
        Version(docker.__version__) < Version("4.0.0"),
        reason="Test does not work in this version of docker-py",
    ),
]


@pytest.fixture
def configure_loader_modules(docker_cmd_run_all_wrapper):
    return {
        rabbitmq_upstream: {
            "__salt__": {
                "rabbitmq.upstream_exists": rabbitmq.upstream_exists,
                "rabbitmq.list_user_permissions": rabbitmq.list_user_permissions,
            },
            "__opts__": {"test": False},
            "_utils__": {},
        },
        rabbitmq: {
            "__salt__": {"cmd.run_all": docker_cmd_run_all_wrapper},
            "__opts__": {},
            "_utils__": {},
        },
    }


def test_absent(rabbitmq_container):
    """
    Test rabbitmq_upstream.absent
    """

    # Delete the user
    ret = rabbitmq_upstream.absent("upstream")
    expected = {
        "name": "upstream",
        "result": True,
        "comment": 'The upstream "upstream" is already absent.',
        "changes": {},
    }
    assert ret == expected



"""
tests.pytests.unit.test_syspaths
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Unit tests for salt's syspaths module
"""

import pytest

import salt.syspaths
from tests.support.mock import patch


@pytest.mark.skip_unless_on_windows(reason="Test is only applicable to Windows.")
def test__get_windows_root_dir_expand_reg():
    return_value = {"success": True, "vdata": "%ProgramData%\\Salt Project\\Salt"}
    expected = "C:\\ProgramData\\Salt Project\\Salt"
    with patch(
        "salt.utils.win_reg.read_value", autospec=True, return_value=return_value
    ):
        result = salt.syspaths._get_windows_root_dir()

    assert expected == result


@pytest.mark.skip_unless_on_windows(reason="Test is only applicable to Windows.")
def test__get_windows_root_dir_no_expand_reg():
    return_value = {"success": True, "vdata": "C:\\ProgramData\\Salt Project\\Salt"}
    expected = "C:\\ProgramData\\Salt Project\\Salt"
    with patch(
        "salt.utils.win_reg.read_value", autospec=True, return_value=return_value
    ):
        result = salt.syspaths._get_windows_root_dir()

    assert expected == result


@pytest.mark.skip_unless_on_windows(reason="Test is only applicable to Windows.")
def test__get_windows_root_dir_no_reg_old_exists():
    return_value = {"success": False, "comment": "Not found"}
    expected = "C:\\salt"
    with patch(
        "salt.utils.win_reg.read_value", autospec=True, return_value=return_value
    ), patch("os.path.isdir", return_value=True):
        result = salt.syspaths._get_windows_root_dir()

    assert expected == result


@pytest.mark.skip_unless_on_windows(reason="Test is only applicable to Windows.")
def test__get_windows_root_dir_no_reg_default():
    return_value = {"success": False, "comment": "Not found"}
    expected = "C:\\ProgramData\\Salt Project\\Salt"
    with patch(
        "salt.utils.win_reg.read_value", autospec=True, return_value=return_value
    ), patch("os.path.isdir", return_value=False):
        result = salt.syspaths._get_windows_root_dir()

    assert expected == result



import pytest

from tests.support.mock import Mock, call, patch

pytest.importorskip("mako")

# This import needs to be after the above importorskip so that no ImportError
# is raised if Mako is not installed
from salt.utils.mako import SaltMakoTemplateLookup


def test_mako_template_lookup(minion_opts):
    """
    The shudown method can be called without raising an exception when the
    file_client does not have a destroy method
    """
    # Test SaltCacheLoader creating and destroying the file client created
    file_client = Mock()
    with patch("salt.fileclient.get_file_client", return_value=file_client):
        loader = SaltMakoTemplateLookup(minion_opts)
        assert loader._file_client is None
        assert loader.file_client() is file_client
        assert loader._file_client is file_client
        try:
            loader.destroy()
        except AttributeError:
            pytest.fail("Regression when calling SaltMakoTemplateLookup.destroy()")
        assert file_client.mock_calls == [call.destroy()]



"""
:codeauthor: Shane Lee <slee@saltstack.com>
"""

import pytest

import salt.modules.cmdmod as cmdmod
import salt.modules.win_file as win_file
import salt.modules.win_lgpo as win_lgpo
import salt.utils.win_dacl as win_dacl
import salt.utils.win_lgpo_auditpol as win_lgpo_auditpol
import salt.utils.win_reg as win_reg

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
    pytest.mark.slow_test,
]


@pytest.fixture
def configure_loader_modules(tmp_path):
    cachedir = tmp_path / "__test_admx_policy_cache_dir"
    cachedir.mkdir(parents=True, exist_ok=True)
    return {
        win_lgpo: {
            "__salt__": {
                "cmd.run": cmdmod.run,
                "file.file_exists": win_file.file_exists,
                "file.makedirs": win_file.makedirs_,
                "file.remove": win_file.remove,
                "file.write": win_file.write,
            },
            "__opts__": {
                "cachedir": str(cachedir),
            },
            "__utils__": {
                "auditpol.get_advaudit_policy_rows": win_lgpo_auditpol.get_advaudit_policy_rows,
                "auditpol.get_auditpol_dump": win_lgpo_auditpol.get_auditpol_dump,
                "reg.read_value": win_reg.read_value,
            },
            "__context__": {},
        },
        win_file: {
            "__utils__": {
                "dacl.set_perms": win_dacl.set_perms,
            },
        },
    }


def _get_policy_adm_setting(
    policy_name, policy_class, return_full_policy_names, hierarchical_return
):
    """
    Helper function to get current setting
    """
    try:
        # Enable the policy
        computer_policy = {
            "Point and Print Restrictions": {
                "Users can only point and print to these servers": True,
                "Enter fully qualified server names separated by semicolons": (
                    "fakeserver1;fakeserver2"
                ),
                "Users can only point and print to machines in their forest": True,
                "When installing drivers for a new connection": (
                    "Show warning and elevation prompt"
                ),
                "When updating drivers for an existing connection": (
                    "Show warning only"
                ),
            },
        }
        win_lgpo.set_(computer_policy=computer_policy)
        # Get the policy
        success, policy_obj, _, _ = win_lgpo._lookup_admin_template(
            policy_name=policy_name, policy_class=policy_class, adml_language="en-US"
        )
        if success:
            results = win_lgpo._get_policy_adm_setting(
                admx_policy=policy_obj,
                policy_class=policy_class,
                adml_language="en-US",
                return_full_policy_names=return_full_policy_names,
                hierarchical_return=hierarchical_return,
            )
            return results
        return "Policy Not Found"
    finally:
        win_lgpo.set_computer_policy(
            name="Point and Print Restrictions",
            setting="Not Configured",
        )


@pytest.mark.slow_test
def test_enabled():
    result = _get_policy_adm_setting(
        policy_name="Point and Print Restrictions",
        policy_class="Machine",
        return_full_policy_names=False,
        hierarchical_return=False,
    )
    expected = {
        "PointAndPrint_Restrictions_Win7": {
            "PointAndPrint_NoWarningNoElevationOnInstall_Enum": (
                "Show warning and elevation prompt"
            ),
            "PointAndPrint_NoWarningNoElevationOnUpdate_Enum": "Show warning only",
            "PointAndPrint_TrustedForest_Chk": True,
            "PointAndPrint_TrustedServers_Chk": True,
            "PointAndPrint_TrustedServers_Edit": "fakeserver1;fakeserver2",
        }
    }
    assert result == expected


def test_enabled_hierarchical():
    result = _get_policy_adm_setting(
        policy_name="Point and Print Restrictions",
        policy_class="Machine",
        return_full_policy_names=False,
        hierarchical_return=True,
    )
    expected = {
        "Computer Configuration": {
            "Administrative Templates": {
                "Printers": {
                    "PointAndPrint_Restrictions_Win7": {
                        "PointAndPrint_NoWarningNoElevationOnInstall_Enum": (
                            "Show warning and elevation prompt"
                        ),
                        "PointAndPrint_NoWarningNoElevationOnUpdate_Enum": (
                            "Show warning only"
                        ),
                        "PointAndPrint_TrustedForest_Chk": True,
                        "PointAndPrint_TrustedServers_Chk": True,
                        "PointAndPrint_TrustedServers_Edit": (
                            "fakeserver1;fakeserver2"
                        ),
                    }
                }
            }
        }
    }
    assert result == expected


def test_enabled_full_names():
    result = _get_policy_adm_setting(
        policy_name="Point and Print Restrictions",
        policy_class="Machine",
        return_full_policy_names=True,
        hierarchical_return=False,
    )
    expected = {
        "Printers\\Point and Print Restrictions": {
            "Enter fully qualified server names separated by semicolons": (
                "fakeserver1;fakeserver2"
            ),
            "When installing drivers for a new connection": (
                "Show warning and elevation prompt"
            ),
            "Users can only point and print to machines in their forest": True,
            "Users can only point and print to these servers": True,
            "When updating drivers for an existing connection": "Show warning only",
        }
    }
    assert result == expected


@pytest.mark.slow_test
def test_full_names_hierarchical():
    result = _get_policy_adm_setting(
        policy_name="Point and Print Restrictions",
        policy_class="Machine",
        return_full_policy_names=True,
        hierarchical_return=True,
    )
    expected = {
        "Computer Configuration": {
            "Administrative Templates": {
                "Printers": {
                    "Point and Print Restrictions": {
                        "Enter fully qualified server names separated by semicolons": (
                            "fakeserver1;fakeserver2"
                        ),
                        "When installing drivers for a new connection": (
                            "Show warning and elevation prompt"
                        ),
                        "Users can only point and print to machines in their forest": True,
                        "Users can only point and print to these servers": True,
                        "When updating drivers for an existing connection": (
                            "Show warning only"
                        ),
                    }
                }
            }
        }
    }
    assert result == expected



"""
    :codeauthor: :email:`Shane Lee <slee@saltstack.com>`
"""

import salt.grains.pending_reboot as pending_reboot
from tests.support.mock import patch


def test_pending_reboot_false():
    with patch("salt.utils.win_system.get_pending_reboot", return_value=False):
        result = pending_reboot.pending_reboot()
        assert result["pending_reboot"] is False


def test_pending_reboot_true():
    with patch("salt.utils.win_system.get_pending_reboot", return_value=True):
        result = pending_reboot.pending_reboot()
        assert result["pending_reboot"] is True



import hashlib
import pathlib
import shutil

import pytest

from tests.support.helpers import Webserver
from tests.support.runtests import RUNTIME_VARS


@pytest.fixture(scope="module")
def file(states):
    return states.file


@pytest.fixture
def grail(state_tree):
    src = pathlib.Path(RUNTIME_VARS.BASE_FILES) / "grail"
    dst = state_tree / "grail"
    try:
        shutil.copytree(str(src), str(dst))
        assert dst.exists()
        yield dst
    finally:
        shutil.rmtree(str(dst), ignore_errors=True)


@pytest.fixture
def holy(state_tree_prod):
    src = pathlib.Path(RUNTIME_VARS.PROD_FILES) / "holy"
    dst = state_tree_prod / "holy"
    try:
        shutil.copytree(str(src), str(dst))
        assert dst.exists()
        yield dst
    finally:
        shutil.rmtree(str(dst), ignore_errors=True)


@pytest.fixture
def grail_scene33_file(grail):
    signed_file = grail / "scene33"
    hash_file = signed_file.with_suffix(".SHA256")
    for test_file in (signed_file, hash_file):
        content = test_file.read_bytes()
        content = content.replace(b"\r\n", b"\n")
        test_file.write_bytes(content)
    return signed_file


@pytest.fixture
def grail_scene33_clearsign_file(grail_scene33_file):
    return grail_scene33_file.with_suffix(".clearsign.asc")


@pytest.fixture
def grail_scene33_file_hash(grail_scene33_file):
    return hashlib.sha256(grail_scene33_file.read_bytes()).hexdigest()


@pytest.fixture
def grail_scene33_clearsign_file_hash(grail_scene33_clearsign_file):
    return hashlib.sha256(grail_scene33_clearsign_file.read_bytes()).hexdigest()


@pytest.fixture(scope="module")
def state_file_account():
    with pytest.helpers.create_account(create_group=True) as system_account:
        yield system_account


@pytest.fixture(scope="module")
def webserver(state_tree):
    _webserver = Webserver(root=str(state_tree))
    try:
        _webserver.start()
        yield _webserver
    finally:
        _webserver.stop()



import subprocess

import pytest
from pytestshellutils.exceptions import FactoryNotStarted
from pytestshellutils.utils.processes import terminate_process

pytestmark = [
    pytest.mark.destructive_test,
    pytest.mark.skip_on_freebsd(reason="Daemons tests are flaky on FreeBSD"),
]


def terminate_daemon(factory):
    # We are going to kill the possible child processes based on the unique config directory
    # We know this is unique to these processes because of the unique name of the config files
    config_dir = factory.config_dir
    pgrep_proc = subprocess.Popen(
        ["pgrep", "-f", str(config_dir)],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    stdout, _ = pgrep_proc.communicate()
    pids = [int(pid) for pid in stdout.decode().strip().split()]
    for pid in pids:
        try:
            terminate_process(pid, kill_children=True)
        except (OSError, RuntimeError):
            pass


@pytest.mark.parametrize("cli_daemon_flag", ["-d", "--daemon"])
def test_salt_master_as_daemon(salt_master_factory, cli_daemon_flag):
    try:
        with salt_master_factory.started(
            cli_daemon_flag, start_timeout=120, max_start_attempts=1
        ):
            pass
    except FactoryNotStarted:
        pass
    finally:
        assert salt_master_factory.impl._terminal_result.stdout == ""
        assert salt_master_factory.impl._terminal_result.stderr == ""
        assert salt_master_factory.impl._terminal_result.returncode == 0
        terminate_daemon(salt_master_factory)


@pytest.mark.parametrize("cli_daemon_flag", ["-d", "--daemon"])
def test_salt_minion_as_daemon(salt_minion_factory, cli_daemon_flag):
    try:
        with salt_minion_factory.started(
            cli_daemon_flag, start_timeout=120, max_start_attempts=1
        ):
            pass
    except FactoryNotStarted:
        pass
    finally:
        assert salt_minion_factory.impl._terminal_result.stdout == ""
        assert salt_minion_factory.impl._terminal_result.stderr == ""
        assert salt_minion_factory.impl._terminal_result.returncode == 0
        terminate_daemon(salt_minion_factory)



import logging
import os

import pytest

import salt.serializers.json as jsonserializer
import salt.serializers.msgpack as msgpackserializer
import salt.serializers.yaml as yamlserializer
import salt.states.file as filestate
import salt.utils.files
import salt.utils.json
import salt.utils.platform
import salt.utils.win_functions
import salt.utils.yaml
from tests.support.mock import MagicMock, mock_open, patch

log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    return {
        filestate: {
            "__env__": "base",
            "__salt__": {"file.manage_file": False},
            "__serializers__": {
                "yaml.serialize": yamlserializer.serialize,
                "yaml.seserialize": yamlserializer.serialize,
                "json.serialize": jsonserializer.serialize,
                "msgpack.serialize": msgpackserializer.serialize,
            },
            "__opts__": {"test": False, "cachedir": ""},
            "__instance_id__": "",
            "__low__": {},
            "__utils__": {},
        }
    }


# 'prepend' function tests: 1
def test_prepend():
    """
    Test to ensure that some text appears at the beginning of a file.
    """
    name = "/tmp/etc/motd"
    if salt.utils.platform.is_windows():
        name = "c:\\tmp\\etc\\motd"
    assert not os.path.exists(os.path.split(name)[0])
    source = ["salt://motd/hr-messages.tmpl"]
    sources = ["salt://motd/devops-messages.tmpl"]
    text = ["Trust no one unless you have eaten much salt with him."]

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    comt = "Must provide name to file.prepend"
    ret.update({"comment": comt, "name": ""})
    assert filestate.prepend("") == ret

    comt = "source and sources are mutually exclusive"
    ret.update({"comment": comt, "name": name})
    assert filestate.prepend(name, source=source, sources=sources) == ret

    mock_t = MagicMock(return_value=True)
    mock_f = MagicMock(return_value=False)
    with patch.dict(
        filestate.__salt__,
        {
            "file.directory_exists": mock_f,
            "file.makedirs": mock_t,
            "file.stats": mock_f,
            "cp.get_template": mock_f,
            "file.search": mock_f,
            "file.prepend": mock_t,
        },
    ):
        comt = "The following files will be changed:\n/tmp/etc: directory - new\n"
        changes = {"/tmp/etc": {"directory": "new"}}
        if salt.utils.platform.is_windows():
            comt = 'The directory "c:\\tmp\\etc" will be changed'
            changes = {"c:\\tmp\\etc": {"directory": "new"}}
        ret.update({"comment": comt, "name": name, "changes": changes})
        assert filestate.prepend(name, makedirs=True) == ret

        with patch.object(os.path, "isabs", mock_f):
            comt = f"Specified file {name} is not an absolute path"
            ret.update({"comment": comt, "changes": {}})
            assert filestate.prepend(name) == ret

        with patch.object(os.path, "isabs", mock_t):
            with patch.object(os.path, "exists", mock_t):
                comt = f"Failed to load template file {source}"
                ret.update({"comment": comt, "name": source, "data": []})
                assert filestate.prepend(name, source=source) == ret

                ret.pop("data", None)
                ret.update({"name": name})
                with patch.object(
                    salt.utils.files, "fopen", MagicMock(mock_open(read_data=""))
                ):
                    with patch.dict(filestate.__utils__, {"files.is_text": mock_f}):
                        with patch.dict(filestate.__opts__, {"test": True}):
                            change = {"diff": "Replace binary file"}
                            comt = f"File {name} is set to be updated"
                            ret.update(
                                {"comment": comt, "result": None, "changes": change}
                            )
                            assert filestate.prepend(name, text=text) == ret

                        with patch.dict(filestate.__opts__, {"test": False}):
                            comt = "Prepended 1 lines"
                            ret.update({"comment": comt, "result": True, "changes": {}})
                            assert filestate.prepend(name, text=text) == ret



"""
Tests for minion blackout
"""

import logging

import pytest

log = logging.getLogger(__name__)


def _check_skip(grains):
    """
    Skip on windows because these tests are flaky, we need to spend some time to
    debug why.
    """
    if grains["os"] == "Windows":
        return True
    return False


pytestmark = [
    pytest.mark.slow_test,
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_initial_gh_actions_failure(skip=_check_skip),
]


def test_blackout(salt_cli, blackout, salt_minion_1):
    """
    Test that basic minion blackout functionality works
    """
    ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
    assert ret.returncode == 0
    assert ret.data is True
    with blackout.enter_blackout("minion_blackout: true"):
        ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
        assert ret.returncode == 1
        assert "Minion in blackout mode." in ret.stdout
    ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
    assert ret.returncode == 0
    assert ret.data is True


def test_blackout_whitelist(salt_cli, blackout, salt_minion_1):
    """
    Test that minion blackout whitelist works
    """
    blackout_contents = """
    minion_blackout: True
    minion_blackout_whitelist:
      - test.ping
      - test.fib
    """
    ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
    assert ret.returncode == 0
    assert ret.data is True
    with blackout.enter_blackout(blackout_contents):
        ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
        assert ret.returncode == 0
        assert ret.data is True

        ret = salt_cli.run("test.fib", "7", minion_tgt=salt_minion_1.id)
        assert ret.returncode == 0
        assert ret.data[0] == 13


def test_blackout_nonwhitelist(salt_cli, blackout, salt_minion_1):
    """
    Test that minion refuses to run non-whitelisted functions during
    blackout whitelist
    """
    blackout_contents = """
    minion_blackout: True
    minion_blackout_whitelist:
      - test.ping
      - test.fib
    """
    ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
    assert ret.returncode == 0
    assert ret.data is True
    with blackout.enter_blackout(blackout_contents):
        ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
        assert ret.returncode == 0
        assert ret.data is True

        ret = salt_cli.run("state.apply", minion_tgt=salt_minion_1.id)
        assert ret.returncode == 1
        assert "Minion in blackout mode." in ret.stdout

        ret = salt_cli.run(
            "cloud.query", "list_nodes_full", minion_tgt=salt_minion_1.id
        )
        assert ret.returncode == 1
        assert "Minion in blackout mode." in ret.stdout
    ret = salt_cli.run("test.ping", minion_tgt=salt_minion_1.id)
    assert ret.returncode == 0
    assert ret.data is True



# Copyright (c) 2016 SUSE Linux LLC
# All Rights Reserved.
#
# Author: Bo Maryniuk <bo@suse.de>

import hashlib
import os

from yum.plugins import TYPE_CORE

CK_PATH = "/var/cache/salt/minion/rpmdb.cookie"
RPM_PATH = "/var/lib/rpm/Packages"

requires_api_version = "2.5"
plugin_type = TYPE_CORE


def _get_mtime():
    """
    Get the modified time of the RPM Database.

    Returns:
        Unix ticks
    """
    return os.path.exists(RPM_PATH) and int(os.path.getmtime(RPM_PATH)) or 0


def _get_checksum():
    """
    Get the checksum of the RPM Database.

    Returns:
        hexdigest
    """
    digest = hashlib.sha256()
    with open(RPM_PATH, "rb") as rpm_db_fh:
        while True:
            buff = rpm_db_fh.read(0x1000)
            if not buff:
                break
            digest.update(buff)
    return digest.hexdigest()


def posttrans_hook(conduit):
    """
    Hook after the package installation transaction.

    :param conduit:
    :return:
    """
    # Integrate Yum with Salt
    if "SALT_RUNNING" not in os.environ:
        with open(CK_PATH, "w") as ck_fh:
            ck_fh.write(f"{_get_checksum()} {_get_mtime()}\n")
